EffectOverview = class(Turbine.UI.Window)
function EffectOverview:Constructor()
    Turbine.UI.Window.Constructor(self)
    self.width = 800
    self.height = 600

    self:SetSize(self.width, self.height)
    self:SetPosition(300, 215)
    self:SetVisible(true)
    self:SetBackColor(Turbine.UI.Color(0.2, 0.2, 0.2))

    self.VisibleChanged = function()
        if self:IsVisible() then
            self:SetZOrder(100)
            local x, y = self:GetMousePosition()
            local oldX, oldY = self:GetPosition()
            self:SetPosition((x + oldX) - 250, y + oldY + 5)
        end
    end

    self.HistoricalEffectsWindowHeaderLabel = Turbine.UI.Label()
    self.HistoricalEffectsWindowHeaderLabel:SetParent(self)
    self.HistoricalEffectsWindowHeaderLabel:SetSize(200 - 2, 20 - 1)
    self.HistoricalEffectsWindowHeaderLabel:SetPosition(1, 1)
    --self.HistoricalEffectsWindowHeaderLabel:SetMouseVisible(false)
    --self.HistoricalEffectsWindowHeaderLabel:SetTextAlignment(Turbine.UI.ContentAlignment.LeftCenter)
    self.HistoricalEffectsWindowHeaderLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.HistoricalEffectsWindowHeaderLabel:SetMultiline(true)
    self.HistoricalEffectsWindowHeaderLabel:SetText("HistoricalEffects selection")

    
    self.Moving = false

    self.HistoricalEffectsWindowHeaderLabel.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.HistoricalEffectsWindowHeaderLabel.MouseUp = function()
        self.Moving = false
    end

    self.HistoricalEffectsWindowHeaderLabel.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self:GetLeft() - (self.MoveX - args.X)
            local newTop = self:GetTop() - (self.MoveY - args.Y)
            self:SetPosition(newLeft,newTop)
        end
    end

    -- Close Button
    self.HistoricalEffectsCloseBackground = Turbine.UI.Control()
    self.HistoricalEffectsCloseBackground:SetParent(self)
    self.HistoricalEffectsCloseBackground:SetPosition(self.width - 20, 0)
    self.HistoricalEffectsCloseBackground:SetSize(20, 20)
    self.HistoricalEffectsCloseBackground:SetMouseVisible(true)
    self.HistoricalEffectsCloseBackground:SetBackColor(Turbine.UI.Color(153/255, 0, 0))


    self.HistoricalEffectsCloseBackground.MouseClick = function()
        self:SetVisible(false)
    end

    self.HistoricalEffectsListBox = Turbine.UI.TreeView()
    self.HistoricalEffectsListBox:SetSize(self.width - 8, self.height - self.HistoricalEffectsWindowHeaderLabel:GetHeight() - 2)
    self.HistoricalEffectsListBox:SetPosition(1, 21)
    self.HistoricalEffectsListBox:SetParent(self)
    self.HistoricalEffectsListBox:SetMouseVisible(false)
    self.HistoricalEffectsListBox:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    --self.HistoricalEffectsListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
    --self.HistoricalEffectsListBox:SetMaxColumns(1)

    self.HistoricalEffectsListBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.HistoricalEffectsListBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.HistoricalEffectsListBoxScrollbar:SetParent(self)
    self.HistoricalEffectsListBoxScrollbar:SetPosition(self.width - 8, 21)
    self.HistoricalEffectsListBoxScrollbar:SetSize(8, self.HistoricalEffectsListBox:GetHeight())
    self.HistoricalEffectsListBox:SetVerticalScrollBar(self.HistoricalEffectsListBoxScrollbar)

    self.rootNodes = self.HistoricalEffectsListBox:GetNodes()

    self.filtering = false

    self.filterFunction = function(object)
        if self.filtering then 
            return not(string.find(string.lower(object.name), string.lower(self.filterKey)))
        else
            return false
        end
    end

    self.sortFunction = function(object1, object2)
        if object1.name >= object2.name then 
            return 1
        else
            return -1
        end
    end

    self.HistoricalEffectsListBox:SetFilter(self.filterFunction)
    self.HistoricalEffectsListBox:SetSortMethod(self.sortFunction)

    self.searchBox = Turbine.UI.Lotro.TextBox()
    self.searchBox:SetParent(self)
    self.searchBox:SetSize(self.width - self.HistoricalEffectsWindowHeaderLabel:GetWidth() - self.HistoricalEffectsCloseBackground:GetWidth(), 19)
    self.searchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBox:SetPosition(self.HistoricalEffectsWindowHeaderLabel:GetWidth(), 1)
    self.searchBox:SetMultiline(false)

    self.filterTimer = 0

    self.updateCallback = function(delta)
        self.filterTimer = self.filterTimer - delta
        if self.filterTimer < 0 then
            if self.callback then 
                RemoveCallback(Updater, "ExecuteUpdate", self.callback) 
                self.callback = nil
            end 
            self.filterKey = self.searchBox:GetText()
            if string.len(self.filterKey) > 0 then
                self.filtering = true
                self.HistoricalEffectsListBox:Refresh()
            else
                self.filtering = false
                self.HistoricalEffectsListBox:Refresh()
            end
            self.searchBox.TextChanged = self.textChangedFunction
        end
    end

    self.textChangedFunction = function()
        self.filterTimer = 3
        if not(self.callback) then
            self.callback = AddCallback(Updater, "ExecuteUpdate", self.updateCallback)
        end
    end

    self.searchBox.TextChanged = self.textChangedFunction

    self.searchBox.KeyDown = function(sender, args)
        if args.Action == 162 and self.searchBox:HasFocus() then
           self.filterTimer = 0 
           self.searchBox.TextChanged = nil
        end
    end

    import "PrimePlugins.RaidTools.Data.EffectData"
	for key, value in pairs(GetEffectData()) do
        if key == "vitalOverTimeEffect" or key == "reactiveVitalEffect" or key == "procEffect" or key == "propertyModEffect" or (tonumber(key) or 0) > 0 then
            self:AddData(value)
        end
	end

    if DataGrabberTool then
        local callbackFunction = function(data)
            local duplicates = {}
            --PrintTable(data)
            for key, value in spairs(data, function(t,a,b) return a < b end) do
                if duplicates[value] then
                else
                    local node = MyTreeNode()
                    node:SetSize(self.HistoricalEffectsListBox:GetWidth(),32)
                    node.name = key

                    local HistoricalEffects = Turbine.UI.Control()
                    HistoricalEffects:SetParent(node)
                    HistoricalEffects:SetSize(self.HistoricalEffectsListBox:GetWidth(),32)
                    HistoricalEffects:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    --HistoricalEffects:SetBackground(value[4])
                    HistoricalEffects:SetMouseVisible(true)

                    HistoricalEffects.MouseEnter = function()
                        HistoricalEffects:SetBackColor(Turbine.UI.Color(0.5,0.5,0.5))
                    end

                    HistoricalEffects.MouseLeave = function()
                        HistoricalEffects:SetBackColor(nil)
                    end

                    local HistoricalEffectsIcon = Turbine.UI.Control()
                    HistoricalEffectsIcon:SetSize(32,32)
                    HistoricalEffectsIcon:SetParent(HistoricalEffects)
                    HistoricalEffectsIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsIcon:SetBackground(BackgroundHandler:GetBackground(value[4]))
                    HistoricalEffectsIcon:SetMouseVisible(false)

                    local HistoricalEffectsName = Turbine.UI.Label()
                    HistoricalEffectsName:SetSize(self.HistoricalEffectsListBox:GetWidth() - 32,32)
                    HistoricalEffectsName:SetParent(HistoricalEffects)
                    HistoricalEffectsName:SetPosition(32, 0)
                    HistoricalEffectsName:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsName:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
                    HistoricalEffectsName:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                    local duration = ""
                    if value[3] < 999999 then
                        duration = " Dur: " .. tostring(value[3]) .. "s"
                    end
                    HistoricalEffectsName:SetText(key .. duration)
                    HistoricalEffectsName:SetMouseVisible(false)


                    local HistoricalEffectsName = Turbine.UI.Label()
                    HistoricalEffectsName:SetSize(self.HistoricalEffectsListBox:GetWidth() - 32,32)
                    HistoricalEffectsName:SetParent(HistoricalEffects)
                    HistoricalEffectsName:SetPosition(32, 0)
                    HistoricalEffectsName:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
                    HistoricalEffectsName:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
                    HistoricalEffectsName:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
                    HistoricalEffectsName:SetText(tostring(value[2]))
                    HistoricalEffectsName:SetMouseVisible(false)
                    HistoricalEffectsName:SetMultiline(false)

                    HistoricalEffects.MouseClick = function()
                        if self.icontext then
                            self.icontext:SetText(key)
                            
                        end
                        self:SetVisible(false)

                    end

                    --self.HistoricalEffectsListBox:AddItem(HistoricalEffects)
                    duplicates[value] = true

                    self.rootNodes:Add(node)


                end
            end
            --self.infoLabel:SetVisible(false)
        end
        DataGrabberTool:GetEffects(callbackFunction)
    end
end

function EffectOverview:Register(icontext)
    self.icontext = icontext
end

function EffectOverview:AddData(data)
	for key, value in spairs(data, function(t,a,b) return a < b end) do
        if tonumber(value.Duration) <= 0 then return end
		local node = MyTreeNode()
		node:SetSize(self.HistoricalEffectsListBox:GetWidth(), 32)
        --node:SetBackColor(Turbine.UI.Color.Black)
		node.name = value.Name

		local HistoricalEffects = Turbine.UI.Control()
		HistoricalEffects:SetParent(node)
		HistoricalEffects:SetSize(self.HistoricalEffectsListBox:GetWidth(), node:GetHeight())
		HistoricalEffects:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
		--HistoricalEffects:SetBackground(value[4])

		HistoricalEffects:SetMouseVisible(true)

		HistoricalEffects.MouseEnter = function()
			HistoricalEffects:SetBackColor(Turbine.UI.Color(0.5,0.5,0.5))
		end

		HistoricalEffects.MouseLeave = function()
			HistoricalEffects:SetBackColor(nil)
		end

		local HistoricalEffectsIcon = Turbine.UI.Control()
		HistoricalEffectsIcon:SetSize(32, 32)
        HistoricalEffectsIcon:SetPosition(0, 0)
		HistoricalEffectsIcon:SetParent(HistoricalEffects)
		HistoricalEffectsIcon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
		HistoricalEffectsIcon:SetBackground(BackgroundHandler:GetBackground(value.Icon))
		HistoricalEffectsIcon:SetMouseVisible(false)

		local HistoricalEffectsName = Turbine.UI.Label()
		HistoricalEffectsName:SetSize(self.HistoricalEffectsListBox:GetWidth() - 32, 16)
		HistoricalEffectsName:SetParent(HistoricalEffects)
		HistoricalEffectsName:SetPosition(32, 0)
		HistoricalEffectsName:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
		HistoricalEffectsName:SetTextAlignment(Turbine.UI.ContentAlignment.BottomLeft)
		HistoricalEffectsName:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
		HistoricalEffectsName:SetText(string.format("%s - %ss", value.Name, value.Duration))
		HistoricalEffectsName:SetMouseVisible(false)
		HistoricalEffectsName:SetMultiline(false)

		local HistoricalEffectsDescription = Turbine.UI.Label()
		HistoricalEffectsDescription:SetSize(self.HistoricalEffectsListBox:GetWidth() - 32, 16)
		HistoricalEffectsDescription:SetParent(HistoricalEffects)
		HistoricalEffectsDescription:SetPosition(32, 16)
		HistoricalEffectsDescription:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
		HistoricalEffectsDescription:SetTextAlignment(Turbine.UI.ContentAlignment.TopLeft)
		HistoricalEffectsDescription:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
		HistoricalEffectsDescription:SetText(string.format("%s %s", value.Description, value["Applied Description"]))
		HistoricalEffectsDescription:SetMouseVisible(false)
		HistoricalEffectsDescription:SetMultiline(false)

		HistoricalEffects.MouseClick = function()
			if self.icontext then
				self.icontext:SetText(value.Name)
				
			end
			self:SetVisible(false)
		end

		self.rootNodes:Add(node)
	end
end