SimpleIconOverview = class(Turbine.UI.Window)
function SimpleIconOverview:Constructor()
    Turbine.UI.Window.Constructor(self)

	self.width = 880
	self.height = 600
	self.borderSize = 1

    self:SetSize(self.width, self.height)
    self:SetPosition(300, 215)
    self:SetVisible(true)
    self:SetBackColor(Turbine.UI.Color(0.2, 0.2, 0.2))

    self.VisibleChanged = function()
        if self:IsVisible() then
            self:SetZOrder(100)
            local x, y = self:GetMousePosition()
            local oldX, oldY = self:GetPosition()
            self:SetPosition((x + oldX) - 250, y + oldY + 5)
        end
    end

    self.IconWindowHeaderLabel = Turbine.UI.Label()
    self.IconWindowHeaderLabel:SetParent(self)
    self.IconWindowHeaderLabel:SetSize(200, 20 - self.borderSize)
    self.IconWindowHeaderLabel:SetPosition(1, 1)
    self.IconWindowHeaderLabel:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.IconWindowHeaderLabel:SetMultiline(true)
    self.IconWindowHeaderLabel:SetText("Icon selection")

	-- Close Button
	self.IconCloseBackground = Turbine.UI.Control()
	self.IconCloseBackground:SetParent(self)
	self.IconCloseBackground:SetPosition(self.width - 20, 0)
	self.IconCloseBackground:SetSize(20, 20)
	self.IconCloseBackground:SetMouseVisible(true)
	self.IconCloseBackground:SetBackColor(Turbine.UI.Color(153/255, 0, 0))

    self.searchBox = Turbine.UI.Lotro.TextBox()
    self.searchBox:SetParent(self)
    self.searchBox:SetSize(self.width - self.IconWindowHeaderLabel:GetWidth() - self.IconCloseBackground:GetWidth(), 20 - self.borderSize)
    self.searchBox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.searchBox:SetPosition(self.IconWindowHeaderLabel:GetWidth(), self.borderSize)
    self.searchBox:SetMultiline(false)


	self.IconCloseBackground.MouseClick = function()
        self:SetVisible(false)
    end
    
    self.Moving = false

    self.IconWindowHeaderLabel.MouseDown = function(sender,args)
        self.MoveX = args.X
        self.MoveY = args.Y
        self.Moving = true
    end

    self.IconWindowHeaderLabel.MouseUp = function()
        self.Moving = false
    end

    self.IconWindowHeaderLabel.MouseMove = function(sender,args)
        if self.Moving then
            local newLeft = self:GetLeft() - (self.MoveX - args.X)
            local newTop = self:GetTop() - (self.MoveY - args.Y)
            self:SetPosition(newLeft,newTop)
        end
    end

    self.iconListBox = Turbine.UI.ListBox()
    self.iconListBox:SetSize(self.width - self.borderSize * 2, self.height - self.IconWindowHeaderLabel:GetHeight() - 2 * self.borderSize)
    self.iconListBox:SetPosition(self.borderSize, self.IconWindowHeaderLabel:GetHeight() + self.borderSize)
    self.iconListBox:SetParent(self)
    self.iconListBox:SetMouseVisible(false)
    self.iconListBox:SetBackColor(Turbine.UI.Color(0.250, 0.250, 0.250))
    self.iconListBox:SetOrientation( Turbine.UI.Orientation.Horizontal )
    self.iconListBox:SetMaxColumns(27)

    self.iconListBoxScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.iconListBoxScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    self.iconListBoxScrollbar:SetParent(self)
    self.iconListBoxScrollbar:SetPosition(self.width - 10, 21)
    self.iconListBoxScrollbar:SetSize(10,  self.height - self.IconWindowHeaderLabel:GetHeight() - 2 * self.borderSize)
    self.iconListBox:SetVerticalScrollBar(self.iconListBoxScrollbar)

	self.searchTime = 0
	self.Update = function()
		if self.searchTime < Turbine.Engine.GetGameTime() then
			self:SetWantsUpdates(false)

			for i = 1, self.iconListBox:GetItemCount(), 1 do
				local object = self.iconListBox:GetItem(i)
				if self.allVisible then
					object:SetVisible(true)
				else
					object:SetVisible(string.find(string.lower(object.name), string.lower(self.filterKey)))
				end
			end
		end
	end

    self.searchBox.TextChanged = function()
		self:SetWantsUpdates(true)
		self.searchTime = Turbine.Engine.GetGameTime() + 2
        self.filterKey = self.searchBox:GetText()
        self.allVisible = string.len(self.filterKey) < 1
    end

    import "PrimePlugins.RaidTools.Data.Icon_Data"
    for category, data in pairs(GetIconSkillData()) do
        self:AddData(data)
    end
    for category, data in pairs(GetIconEffectData()) do
        self:AddData(data)
    end
end

function SimpleIconOverview:AddData(data)
    for key, value in spairs(data, function(t, a, b) return a < b end) do
        if value.Icon and value.Icon ~= 1090522259 then
            local icon = Turbine.UI.Control()
            icon:SetSize(32,32)
            icon:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend)
            icon:SetBackground(value.Icon)
            icon:SetMouseVisible(true)

            icon.MouseClick = function()
                if self.iconBox then
                    self.iconBox:SetBackground(BackgroundHandler:GetBackground(value.Icon))
                    self.icontext:SetText(value.Icon)
                end
                ChatOutput(string.format("Selected Icon ID: %s", value.Icon))
            end
            icon.name = value.Name
            self.iconListBox:AddItem(icon)
        end
    end
end

function SimpleIconOverview:Register(iconbox, icontext)
    self.iconBox = iconbox
    self.icontext = icontext
end