GroupFrameHandler = class( Turbine.UI.Window)
function GroupFrameHandler:Constructor(dataHandler)
    Turbine.UI.Window.Constructor(self)
    self:SetMouseVisible(false)

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.handler = dataHandler

    self.Party = self.player:GetParty()

    self.testMode = false
    --if true then return end
    self.VitalWindows = {}
    self.callbacks = {}

    self.partyCount = 12

    self:SetVisible(true)
    self:SetOpacity(1)
    --self:SetBackColor(Turbine.UI.Color(0,0,0))

    self.frameSettings = self.handler:GetActiveProfile()

    self.playerFrameTable = {}

    self.partyFrame = Turbine.UI.ListBox()
    self.partyFrame:SetParent(self)
    self.partyFrame:SetAllowDrop(true)
    self.partyFrame:SetMouseVisible(false)

    if self.Party == nil and not self.testMode then return end 
    --self:BuildFrames()
    --self.Setup = Settings.FrameVersion

    self.partyLeader = self.player:GetName()

    if not self.testMode and self.Party then
        self.partyLeader= self.Party:GetLeader():GetName()
    end


    --self:SetStretchMode(1)
    --self:SetSize(width * 1.5, height * 1.5)

    self:ReloadFrames(self.frameSettings)
end

function GroupFrameHandler:FireEvent(source, value, args)
    --["Highlight_On"] = valuebar.partyHighlightOnCheckbox and valuebar.partyHighlightOnCheckbox:IsChecked(),
    --["Highlight_On_Time"] = valuebar.partyHighlightOnTextbox and tonumber(valuebar.partyHighlightOnTextbox:GetText()) or 0,
    --["Highlight_Off"] = valuebar.partyHighlightOffCheckbox and valuebar.partyHighlightOffCheckbox:IsChecked()

    --ChatOutput(source)

    if value.Party_Leader and self.partyLeader ~= source then
        return
    end

    if value.All_Party_Members or self.testMode then
    --if value.All_Party_Members then
        for i = 1, self.partyFrame:GetItemCount() do
            self:FireEventInternal(self.partyFrame:GetItem(i), value, args)
        end
    elseif value.Use_Match_As_Target then
        for argsKey, name in pairs(args) do
            if string.find(argsKey, "Match") and self.playerFrameTable[name] then
                self:FireEventInternal(self.playerFrameTable[source], value, args)
            end 
        end
    else
        if self.playerFrameTable[source] then
            self:FireEventInternal(self.playerFrameTable[source], value, args)
        end
    end

end

function GroupFrameHandler:FireEventInternal(frame, value, args)
    if value.Highlight_On then
        frame.highlight:Toggle(true, value.Highlight_On_Time)
    end

    if value.Highlight_Off then
        frame.highlight:ToggleHighlight(false)
    end

    if value.Change_Icon_Status then
        if value.Icon_Off then 
            frame:TurnIconOff()
        else
            frame:SetIconStatus(value.Icon_Type)
        end
    end

    if value.Start_Box_Timer then
        frame:StartBoxTimer(value.Box_Timer_Position, value.Box_Timer_Time)
    end
end

function GroupFrameHandler:SetTestMode(value)
    self.testMode = value
    self:ReloadFrames(self.frameSettings)
end

function GroupFrameHandler:BuildFrames()
    if not Settings.GroupVitalsEnabled then return end
    Turbine.UI.Lotro.LotroUI.SetEnabled(Turbine.UI.Lotro.LotroUIElement.Party, self.frameSettings.Group_Frame.Disable_Base_Frame)

    if not self.testMode and not self.Party then return end

    if self.testMode then
        self.partyCount = 6
    elseif self.Party then
        self.partyCount = self.Party:GetMemberCount()
    end

    for i = 1, self.testMode and self.partyCount or self.Party:GetMemberCount(), 1 do
        if self.testMode then
            self.partyFrame:AddItem(PlayerWindow(self.player, self.frameSettings, self.testMode))
        else
            if self.Party then
                local playerFrame = PlayerWindow(self.Party:GetMember(i), self.frameSettings, self.testMode)
                self.playerFrameTable[self.Party:GetMember(i):GetName()] = playerFrame
                self.partyFrame:AddItem(playerFrame)

            end
        end
    end
end

function GroupFrameHandler:ToggleMove()
    if not self.moveBox then
        self.moveBox = MoveBox(self, self:GetWidth(), self:GetHeight(), "Group Frame")
    end

    local X, Y = self:GetPosition()
    GroupVitalsHandlerObject:ChangeValue(GroupVitalsHandlerObject:GetActiveProfileName(), "Group_Frame", "X", X)
    GroupVitalsHandlerObject:ChangeValue(GroupVitalsHandlerObject:GetActiveProfileName(), "Group_Frame", "Y", Y)

    self.moveBox:ToggleMove()
end

function GroupFrameHandler:UnloadFrames()
    for i = 1, self.partyFrame:GetItemCount() do
        self.partyFrame:GetItem(i).effectHandler:Unload()
    end
    self.playerFrameTable = nil
    self.partyFrame:ClearItems()
    collectgarbage()
    self.playerFrameTable = {}
    Turbine.UI.Lotro.LotroUI.SetEnabled(Turbine.UI.Lotro.LotroUIElement.Party, true)
end

function GroupFrameHandler:ReloadFrames(settings)
    if settings then self.frameSettings = settings end
    self:UnloadFrames()

    local width =  0
    local height = 0
    if self.frameSettings.Group_Frame.Orientation == "Vertical" then
        width = (self.frameSettings.Health.HealthWidth + 2 * self.frameSettings.Group_Frame.BorderSize + self.frameSettings.Group_Frame.BoxOffset) * math.ceil(self.partyCount/self.frameSettings.Group_Frame.Frames_Per_Column) 
        height = (self.frameSettings.Health.HealthHeight + self.frameSettings.Power.PowerHeight + 2 * self.frameSettings.Group_Frame.BorderSize + self.frameSettings.Group_Frame.BoxOffset) * (math.max(1, math.min(self.frameSettings.Group_Frame.Frames_Per_Column, self.partyCount)))
    else
        width = (self.frameSettings.Health.HealthWidth + 2 * self.frameSettings.Group_Frame.BorderSize + self.frameSettings.Group_Frame.BoxOffset)  * (math.max(1, self.frameSettings.Group_Frame.Frames_Per_Column))
        height = (self.frameSettings.Health.HealthHeight + self.frameSettings.Power.PowerHeight + 2 * self.frameSettings.Group_Frame.BorderSize + self.frameSettings.Group_Frame.BoxOffset) * math.ceil(self.partyCount/ math.min(self.frameSettings.Group_Frame.Frames_Per_Column, self.partyCount))
    end
      
    self:SetPosition(self.frameSettings.Group_Frame.X, self.frameSettings.Group_Frame.Y)
    self:SetSize(width, height)

    self.partyFrame:SetSize(width, height)
    self.partyFrame:SetMaxItemsPerLine(self.frameSettings.Group_Frame.Frames_Per_Column)
    self.partyFrame:SetOrientation(Turbine.UI.Orientation[self.frameSettings.Group_Frame.Orientation])
    self.partyFrame:SetReverseFill(self.frameSettings.Group_Frame.Reversefill)
    self.partyFrame:SetFlippedLayout(self.frameSettings.Group_Frame.FlippedLayout)

    self:BuildFrames()
end