ConditionHandler = class()
function ConditionHandler:Constructor()
    self.ActionEvent = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.playerClass = self.player:GetClass()

    self.raceObjectsToName = {
        [Turbine.Gameplay.Race.Beorning] = "Beorning",
        [Turbine.Gameplay.Race.Dwarf] = "Dwarf",
        [Turbine.Gameplay.Race.Elf] = "Elf",
        [Turbine.Gameplay.Race.HighElf] = "HighElf",
        [Turbine.Gameplay.Race.Hobbit] = "Hobbit",
        [Turbine.Gameplay.Race.Man] = "Man",
        [Turbine.Gameplay.Race.StoutAxe] = "StoutAxe",
    } 

    self.compareFunctions = {
        ["<"] = function(percentage, value) return percentage < value end,
        ["<="] = function(percentage, value) return percentage <= value end,
        [">="] = function(percentage, value) return percentage >= value end,
        [">"] = function(percentage, value) return percentage > value end,
    }

    self.conditionTables = {}

    self.combatCallbackCount = 0
    self.combatCallbackIn = {}
    self.combatCallbackOut = {}

    self.mountedCallbackCount = 0
    self.mountedData = {}
    self.mountedCallback = {}
    self.nonMountedCallback = {}

    self.effectCallbackCount = 0
    self.effectTables = {}

    self.locationCallbackCount = 0
    self.locationTables = {}

    self.moraleCallbackCount = 0
    self.moraleThresholdTables = {}

    self.powerCallbackCount = 0
    self.powerThresholdTables = {}

    self.eventCallbackCount = 0
    self.eventItemTable = {}
    self.eventArgs = {}
    self.eventArgsConditionName = {}

    self.eventVariableCallbackCount = 0
    self.eventVariableTable = {}

    self.fightCallbackCount = 0
    self.fightData = {}

    self.fightPhaseCallbackCount = 0
    self.fightPhaseData = {}

    self.statusCallbackCount = 0
    self.statusData = {}

    self.resourceCallbackCount = 0
    self.resourceData = {}

    self.attributeCallbackCount = 0
    self.attrCallbacks = {}
    
    self.levelCallbackCount = 0
    self.levelThresholdTables = {}

    self.lockCallbackCount = 0
    self.lockTable = {}

    self.questCallbackCount = 0
    self.questTable = {}

    self.updateCount = 0
    self.updateCallback = function(delta)
        for conditionName, conditionList in pairs(self.conditionTables) do
            for key, value in pairs(conditionList.SkillItems) do
                if value < Turbine.Engine.GetGameTime() then
                    value = nil
                    if self.conditionTables[conditionName].SkillValue[key] then
                        self.conditionTables[conditionName].StatusTable[key] = 0
                    else
                        self.conditionTables[conditionName].StatusTable[key] = 1
                    end
                    self.updateCount = self.updateCount - 1
                    self:CheckConditions(conditionName)
                end
            end

            for key, value in pairs(conditionList.EventItems) do
                if value < Turbine.Engine.GetGameTime() then
                    value = nil
                    if self.conditionTables[conditionName].SkillValue[key] then
                        self.conditionTables[conditionName].StatusTable[key] = 1
                    else
                        self.conditionTables[conditionName].StatusTable[key] = 0
                    end

                    self.updateCount = self.updateCount - 1
                    self:CheckConditions(conditionName)
                end
            end
        end

        if self.updateCount == 0 then
            RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback)
        end
    end

    self.checkCallback = function()
        for key, _ in pairs(self.conditionsToBeChecked) do
            local tableValue = 0
            for conditionKey, value in pairs(self.conditionTables[key].StatusTable) do
                if value > 0 then
                    if self.conditionTables[key].ApplicationTable[conditionKey] then
                        if value >= self.conditionTables[key].ApplicationTable[conditionKey] then
                            tableValue = tableValue + 1
                        end
                    else
                        tableValue = tableValue + 1
                    end
    
                end
            end

            local value = tableValue >= self.conditionTables[key].Threshold
            --ChatOutput(value)
            --ChatOutput(tableValue)
            --ChatOutput(self.conditionTables[key].Threshold)
            --PrintTable(self.conditionTables[key].StatusTable)
            if value ~= self.conditionTables[key].ConditionValue then
                self.conditionTables[key].ConditionValue = value
                --ChatOutput(key)
                -- TRIGGER    
                if self.conditionTables[key].EventFunction then
                    self.conditionTables[key].EventFunction(value)    
                end
            end
    
            self.conditionsToBeChecked[key] = nil
        end
        self.checking = false
        RemoveCallback(Updater, "ExecuteUpdate", self.checkCallback) 
    end

    self.conditionsToBeChecked = {}
end

function ConditionHandler:FireTrigger(event, args, conditionName)
    self.eventArgs[event] = args
    self.eventArgsConditionName[event] = conditionName
    self.ActionEvent(self, self.EventTrigger, event)
end

function ConditionHandler:AddCondition(conditionTable, func)  
    if self.conditionTables[conditionTable.Name] then
        ThrowException("Failed to add condition table. A table by this name already exists!")
    else
        self.conditionTables[conditionTable.Name] = {}
        self.conditionTables[conditionTable.Name].StatusTable = {}
        self.conditionTables[conditionTable.Name].ApplicationTable = {}
        self.conditionTables[conditionTable.Name].Threshold = conditionTable.Threshold
        self.conditionTables[conditionTable.Name].ConditionInfo = conditionTable
        self.conditionTables[conditionTable.Name].SkillValue = {}
        self.conditionTables[conditionTable.Name].SkillItems = {}
        self.conditionTables[conditionTable.Name].SkillCallbacks = {}
        self.conditionTables[conditionTable.Name].EventItems = {}
        self.conditionTables[conditionTable.Name].ConditionValue = nil
        self.conditionTables[conditionTable.Name].EventFunction = func

        for key, value in pairs(conditionTable.Conditions) do
            if key == "Out_Of_Combat" or key == "In_Combat" then
                self:AddCombatCallback(conditionTable.Name, key, value)
            elseif key == "Effects" then
                self:AddEffectCallbacks(conditionTable.Name, key, value)
            elseif key == "Skill_Ready" then
                self:AddSkillReadyCallback(conditionTable.Name, key, value)
            elseif key == "Mounted" or key == "Non_Mounted" then
                self:AddMountCallback(conditionTable.Name, key, value)
            elseif key == "Location" then
                self:AddLocationCallback(conditionTable.Name, key, value)
            elseif key == "Morale" then
                self:AddMoraleCallback(conditionTable.Name, key, value)
            elseif key == "Power" then
                self:AddPowerCallback(conditionTable.Name, key, value)
            elseif key == "EventTrigger" then
                self:AddEventCallback(conditionTable.Name, key, value)
            elseif key == "EventVariable" then
                self:AddEventVariableCallback(conditionTable.Name, key, value)
            elseif key == "Fight" then
                self:AddFightCallback(conditionTable.Name, key, value)
            elseif key == "FightPhase" then
                self:AddFightPhaseCallback(conditionTable.Name, key, value)
            elseif key == "Status" then
                self:AddStatusCallback(conditionTable.Name, key, value)
            elseif key == "Resource" then
                self:AddResourceCallback(conditionTable.Name, key, value)
            elseif key == "Level" then
                self:AddLevelCallback(conditionTable.Name, key, value)
            elseif key == "Lock" then
                self:AddLootCallback(conditionTable.Name, key, value)
            elseif key == "Quest" then
                self:AddQuestCallback(conditionTable.Name, key, value)
            -- Non call back conditions
            elseif key == "Class" then
                self:HandleClassConditions(conditionTable.Name, key, value)
            elseif key == "Role" then
                self:HandleRoleConditions(conditionTable.Name, key, value)
            elseif key == "Line" then
                self:HandleLineConditions(conditionTable.Name, key, value)
            elseif key == "Race" then
                self:HandleRaceConditions(conditionTable.Name, key, value)
            end
        end

        self:CheckConditions(conditionTable.Name)
    end
end

function ConditionHandler:RemoveCondition(conditionName)
    if not self.conditionTables[conditionName] then
        ThrowException("Failed to remove condition table. A table by this name does not exists!")
    else
        for key, value in pairs(self.conditionTables[conditionName].ConditionInfo.Conditions) do
            if key == "Out_Of_Combat" or key == "In_Combat" then
                self:RemoveCombatCallback(conditionName, key, value)
            elseif key == "Effects" then
                self:RemoveEffectCallbacks(conditionName)
            elseif key == "Skill_Ready" then
                self:RemoveSkillCallback(conditionName)
            elseif key == "Mounted" or key == "Non_Mounted" then
                self:RemoveMountCallback(conditionName, key, value)
            elseif key == "Location" then
                self:RemoveLocationCallback(conditionName, key, value)
            elseif key == "Fight" then
                self:RemoveFightCallback(conditionName, key, value)
            elseif key == "FightPhase" then
                self:RemoveFightPhaseCallback(conditionName, key, value)
            elseif key == "Morale" then
                self:RemoveMoraleCallback(conditionName, key, value)
            elseif key == "Power" and self.player:GetClass() ~= Turbine.Gameplay.Class.Beorning then
                self:RemovePowerCallback(conditionName, key, value)
            elseif key == "EventTrigger" then
                self:RemoveEventCallback(conditionName, key, value)
            elseif key == "EventVariable" then
                self:RemoveEventVariableCallback(conditionName, key, value)
            elseif key == "Status" then
                self:RemoveStatusCallback(conditionName, key, value)
            elseif key == "Resource" then
                self:RemoveResourceCallback(conditionName, key, value)
            elseif key == "Level" then
                self:RemoveLevelCallback(conditionName, key, value)
            elseif key == "Quest" then
                self:RemoveQuestCallback(conditionName, key, value)
            end
        end

        self.conditionTables[conditionName].StatusTable = nil
        self.conditionTables[conditionName].ApplicationTable = nil
        self.conditionTables[conditionName].Threshold = nil
        self.conditionTables[conditionName].ConditionInfo = nil
        self.conditionTables[conditionName].SkillValue = nil
        self.conditionTables[conditionName].SkillItems = nil
        self.conditionTables[conditionName].SkillCallbacks = nil
        self.conditionTables[conditionName].EventItems = nil
        self.conditionTables[conditionName] = nil
    end
end

function ConditionHandler:CheckConditions(conditionName)
    if conditionName then
        self.conditionsToBeChecked[conditionName] = true
    end

    if not self.checking then
        self.checking = true
        AddCallback(Updater, "ExecuteUpdate", self.checkCallback)
    end
end

-- Skill Ready Handling
function ConditionHandler:AddSkillReadyCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do
        if SkillEventHandler:SkillExists(key) then
            if SkillEventHandler:GetSkill(key) then
                local tableName = string.format("Skill_Ready%s", SkillEventHandler:GetSkill(key):GetSkillInfo():GetName())
                self.conditionTables[conditionName].SkillValue[tableName] = value.Not_Applied
                if SkillEventHandler:GetSkill(key):GetResetTime() == -1 then
                    if value.Not_Applied then
                        self.conditionTables[conditionName].StatusTable[tableName] = 0
                    else
                        self.conditionTables[conditionName].StatusTable[tableName] = 1
                    end
                else
                    if value.Not_Applied then
                        self.conditionTables[conditionName].StatusTable[tableName] = 1
                    else
                        self.conditionTables[conditionName].StatusTable[tableName] = 0
                    end

                    self.conditionTables[conditionName].SkillItems[tableName] = SkillEventHandler:GetSkill(key):GetResetTime()
                    if self.updateCount == 0 then
                        AddCallback(Updater, "ExecuteUpdate", self.updateCallback)
                    end
                    self.updateCount = self.updateCount + 1
                end
            end

            self.conditionTables[conditionName].SkillCallbacks[key] = AddCallback(SkillEventHandler, key, function(skill) 
                local tableName = string.format("Skill_Ready%s", skill:GetSkillInfo():GetName())

                local resetTime = skill:GetResetTime()

                -- Skill is ready
                if resetTime == -1  then
                    -- We want to see it when it is on cooldown
                    if value.Not_Applied then
                        -- Set status to 0
                        self.conditionTables[conditionName].StatusTable[tableName] = 0

                        -- If it was previously on cd we need to adjust
                        if self.conditionTables[conditionName].SkillItems[tableName] then
                            self.conditionTables[conditionName].SkillItems[tableName] = nil
                            self.updateCount = self.updateCount - 1
                            if self.updateCount == 0 then
                                RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) 
                            end
                        end
                    else
                        -- Not on cd and we want to see it now
                        self.conditionTables[conditionName].StatusTable[tableName] = 1
                    end
                else
                    -- Skill is on cooldown
                    if value.Not_Applied then
                        -- We want to see it
                        self.conditionTables[conditionName].StatusTable[tableName] = 1
                    else
                        -- We do not want to see it
                        self.conditionTables[conditionName].StatusTable[tableName] = 0
                    end
                    
                    -- Set update procedure to track when we need to adjust
                    self.conditionTables[conditionName].SkillItems[tableName] = resetTime
                    if self.updateCount == 0 then
                        AddCallback(Updater, "ExecuteUpdate", self.updateCallback)
                    end
                    self.updateCount = self.updateCount + 1

                end
                self.conditionsToBeChecked[conditionName] = true
                self:CheckConditions()
            end)
        end
    end
end

function ConditionHandler:RemoveSkillCallback(conditionName)
    for key, value in pairs(self.conditionTables[conditionName].SkillCallbacks) do
        RemoveCallback(SkillEventHandler, key, value)
    end

    for _, _ in pairs(self.conditionTables[conditionName].SkillItems) do
        self.updateCount = self.updateCount - 1
    end

    if self.updateCount == 0 then
        RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback)
    end
end

-- Effect handling
function ConditionHandler:AddEffectCallbacks(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do
        self.conditionTables[conditionName].ApplicationTable[string.format("Effects%s", key)] = value.Applications
        if value.Not_Applied == true then
            self.conditionTables[conditionName].StatusTable[string.format("Effects%s", key)] = 1
        else
            self.conditionTables[conditionName].StatusTable[string.format("Effects%s", key)] = 0
        end

        if not self.effectTables[key] then
            self.effectTables[key] = {}
        end

        self.effectTables[key][conditionName] = value.Not_Applied
    end

    for i = 1, self.player:GetEffects():GetCount(), 1 do
        local effect = self.player:GetEffects():Get(i)
        if self.effectTables[effect:GetName()] then
            local value = self.effectTables[effect:GetName()][conditionName]
            if value ~= nil then
                if value == true then
                    self.conditionTables[conditionName].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[conditionName].StatusTable[string.format("Effects%s", effect:GetName())] - 1
                else
                    self.conditionTables[conditionName].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[conditionName].StatusTable[string.format("Effects%s", effect:GetName())] + 1
                end
            end
        end
    end

    self:CheckConditions(conditionName)

    if self.effectCallbackCount == 0 then
        self.EffectAddedCallback = AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local effectValues = self.effectTables[effect:GetName()]
            if effectValues then
                for key, value in pairs(effectValues) do
                    if value == true then
                        self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] - 1
                    else
                        self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] + 1
                    end
                    self:CheckConditions(key)
                end
            end
        end)

        self.EffectRemovedCallback = AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local effectValues = self.effectTables[effect:GetName()]
            if effectValues then
                for key, value in pairs(effectValues) do
                    if value == true then
                        self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] + 1
                    else
                        self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] =  self.conditionTables[key].StatusTable[string.format("Effects%s", effect:GetName())] - 1
                    end
                    self:CheckConditions(key)
                end

            end
        end)
    end

    self.effectCallbackCount = self.effectCallbackCount + 1
end

function ConditionHandler:RemoveEffectCallbacks(conditionName)
    self.effectCallbackCount = self.effectCallbackCount - 1

    for key, value in pairs(self.effectTables) do
        for effectConditionName, _ in pairs(value) do
            if conditionName == effectConditionName then
                self.effectTables[key][effectConditionName] = nil
                if Tablelength(self.effectTables[key]) == 0 then
                    self.effectTables[key] = nil
                end
            end
        end
    end

    if self.effectCallbackCount == 0 and self.EffectAddedCallback then
        RemoveCallback(self.player, "EffectAdded", self.EffectAddedCallback)
        self.EffectAddedCallback = nil
    end

    if self.effectCallbackCount == 0 and self.EffectRemovedCallback then
        RemoveCallback(self.player, "EffectRemoved", self.EffectRemovedCallback)
        self.EffectRemovedCallback = nil
    end
end

-- Status handling
function ConditionHandler:AddStatusCallback(conditionName, conditionType, conditionInfo)
    if not self.statusData[conditionName] then self.statusData[conditionName] = {} end
    for key, value in pairs(conditionInfo) do
        if not self.statusData[key] then self.statusData[key] = {} end
        self.statusData[key][conditionName] = true
    end

    if self.statusCallbackCount == 0 then
        self.statusAddedCallback = AddCallback(PlayerHandlerObject, "CategoryAdded", function(args)
            if self.statusData[args.Type] then
                for key, _ in pairs(self.statusData[args.Type]) do
                    self.conditionTables[key].StatusTable[string.format("CategoryStatus%s", args.Type)] = 1
                    self:CheckConditions(key)
                end
            end
        end)

        self.statusRemovedCallback = AddCallback(PlayerHandlerObject, "CategoryRemoved", function(args)
            if self.statusData[args.Type] then
                for key, _ in pairs(self.statusData[args.Type]) do
                    self.conditionTables[key].StatusTable[string.format("CategoryStatus%s", args.Type)] = 0
                    self:CheckConditions(key)
                end
            end
        end)
    end

    self.statusCallbackCount = self.statusCallbackCount + 1
end

function ConditionHandler:RemoveStatusCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.statusData) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.statusData[key][key2] = nil
            end
        end
    end

    self.statusCallbackCount = self.statusCallbackCount - 1

    if self.statusCallbackCount == 0 then
        if self.statusAddedCallback then
            RemoveCallback(PlayerHandler, "CategoryAdded", self.statusAddedCallback)
            self.statusAddedCallback = nil
        end

        if self.statusRemovedCallback then
            RemoveCallback(PlayerHandler, "CategoryRemoved", self.statusRemovedCallback)
            self.statusRemovedCallback = nil
        end
    end
end


-- Location handling
function ConditionHandler:AddLocationCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do
        if not self.locationTables[key] then self.locationTables[key] = {}  end
        self.locationTables[key][conditionName] = true
    end

    if self.locationCallbackCount == 0 then
        self.locationCallback = AddCallback(LocationHandler, "Location", function(args)
            local result = self.locationTables[args.Location]
            if result then
                for key, value in pairs(result) do
                    if self.locationTables[args.Location][key] then
                        self.conditionTables[key].StatusTable["Location"] = 1
                        self:CheckConditions(key)
                    else
                        self.conditionTables[key].StatusTable["Location"] = 0
                        self:CheckConditions(key)
                    end
                end

            end

        end)
    end

    self.locationCallbackCount = self.locationCallbackCount + 1

    self.conditionTables[conditionName].StatusTable["Location"] = self.locationTables[LocationHandler:GetCurrentLocation()] and self.locationTables[LocationHandler:GetCurrentLocation()][conditionName] and 1 or 0
end

function ConditionHandler:RemoveLocationCallback(conditionName, conditionType, conditionInfo)
    self.locationCallbackCount = self.locationCallbackCount - 1

    if self.locationCallbackCount == 0 and self.locationCallback then
        RemoveCallback(LocationHandler, "Location", self.locationCallback)
        self.locationCallback = nil
    end
end

-- Combat handling
function ConditionHandler:AddCombatCallback(conditionName, conditionType, conditionInfo)
    if self.combatCallbackCount == 0 then
        self.combatCallback = AddCallback(self.player, "InCombatChanged", function()
            self:InCombatFunction()
            self:OutCombatFunction()
        end)
    end
    
    if conditionType == "Out_Of_Combat" then
        self.conditionTables[conditionName].StatusTable["Must_Be_Out_Of_Combat"] = not(self.player:IsInCombat()) and 1 or 0
        self.combatCallbackOut[conditionName] = true
    elseif conditionType == "In_Combat" then
        self.conditionTables[conditionName].StatusTable["Must_Be_In_Combat"] = self.player:IsInCombat() and 1 or 0
        self.combatCallbackIn[conditionName] = true
    end

    self.combatCallbackCount = self.combatCallbackCount + 1
end

function ConditionHandler:RemoveCombatCallback(conditionName, conditionType, conditionInfo)
    if conditionType == "Out_Of_Combat" then
        self.combatCallbackOut[conditionName] = nil
    elseif conditionType == "In_Combat" then
        self.combatCallbackIn[conditionName] = nil
    end

    self.combatCallbackCount = self.combatCallbackCount - 1

    if self.combatCallbackCount == 0 and self.combatCallback then
        RemoveCallback(self.player, "InCombatChanged", self.combatCallback)
        self.combatCallback = nil
    end
end

function ConditionHandler:InCombatFunction()
    local combatValue = self.player:IsInCombat()
    for key, value in pairs(self.combatCallbackIn) do
        if combatValue then
            self.conditionTables[key].StatusTable["Must_Be_In_Combat"] = 1
        else
            self.conditionTables[key].StatusTable["Must_Be_In_Combat"] = 0
        end
        self:CheckConditions(key)
    end
end

function ConditionHandler:OutCombatFunction()
    local combatValue = self.player:IsInCombat()
    for key, value in pairs(self.combatCallbackOut) do
        if combatValue then
            self.conditionTables[key].StatusTable["Must_Be_Out_Of_Combat"] = 0
        else
            self.conditionTables[key].StatusTable["Must_Be_Out_Of_Combat"] = 1
        end
        self:CheckConditions(key)
    end
end

-- Mount handling
function ConditionHandler:AddMountCallback(conditionName, conditionType, conditionInfo)

    if self.mountedCallbackCount == 0 then
        self.mountCallback = AddCallback(self.player, "MountChanged", function()
            self:MountedFunction()
            self:NonMountedFunction()
        end)
    end
    
    if conditionType == "Not_Mounted" then
        self.conditionTables[conditionName].StatusTable["Not_Mounted"] = not(self.player:GetMount()) and 1 or 0
        self.nonMountedCallback[conditionName] = true
    elseif conditionType == "Mounted" then
        self.conditionTables[conditionName].StatusTable["Mounted"] = self.player:GetMount() and 1 or 0
        self.mountedCallback[conditionName] = true
    end

    self.mountedCallbackCount = self.mountedCallbackCount + 1
end

function ConditionHandler:RemoveMountCallback(conditionName, conditionType, conditionInfo)
    if conditionType == "Not_Mounted" then
        self.nonMountedCallback[conditionName] = nil
    elseif conditionType == "Mounted" then
        self.mountedCallback[conditionName] = nil
    end

    self.mountedCallbackCount = self.mountedCallbackCount - 1

    if self.mountedCallbackCount == 0 and self.mountCallback then
        RemoveCallback(self.player, "MountChanged", self.mountCallback)
        self.mountCallback = nil
    end
end

function ConditionHandler:MountedFunction()
    local combatValue = self.player:GetMount()
    for key, value in pairs(self.mountedCallback) do
        if combatValue then
            self.conditionTables[key].StatusTable["Mounted"] = 1
        else
            self.conditionTables[key].StatusTable["Mounted"] = 0
        end
        self:CheckConditions(key)
    end
end

function ConditionHandler:NonMountedFunction()
    local combatValue = self.player:GetMount()
    for key, value in pairs(self.nonMountedCallback) do
        if combatValue then
            self.conditionTables[key].StatusTable["Not_Mounted"] = 0
        else
            self.conditionTables[key].StatusTable["Not_Mounted"] = 1
        end
        self:CheckConditions(key)
    end
end

-- Event handling
function ConditionHandler:AddEventCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do
        local tableName = string.format("EvenTrigger|%s", value.Variable_Name)
        self.conditionTables[conditionName].SkillValue[tableName] = value.Not_Applied

        if value.Not_Applied then
            self.conditionTables[conditionName].StatusTable[tableName] = 1
        else
            self.conditionTables[conditionName].StatusTable[tableName] = 0
        end
    
        if not self.eventItemTable[key] then
            self.eventItemTable[key] = {}
        end
    
        if not self.eventItemTable[key][conditionName] then
            self.eventItemTable[key][conditionName] = {}
        end
    
        self.eventItemTable[key][conditionName][tableName] = value.Timer 
    end

    if self.eventCallbackCount == 0 then
        self.eventTriggerCallbacks = AddCallback(self, "EventTrigger", 
        function(event)
            local value = self.eventItemTable[event]
            if value then
                for conditionTableName, conditionNameTable in pairs(value) do
                    if self.eventArgsConditionName[event] == conditionTableName and conditionNameTable then
                        for tName, eventDataTableTime in pairs(conditionNameTable) do
                            if self.conditionTables[conditionTableName].SkillValue[tName] then
                                self.conditionTables[conditionTableName].StatusTable[tName] = 0
                            else
                                self.conditionTables[conditionTableName].StatusTable[tName] = 1
                            end

                            local timer = 0
                            if eventDataTableTime == true then
                                timer = tonumber(self.eventArgs[event].Timer) or 0
                            else
                                timer = eventDataTableTime
                            end
                            self.conditionTables[conditionTableName].EventItems[tName] = Turbine.Engine.GetGameTime() +  timer

                            if self.updateCount == 0 then
                                AddCallback(Updater, "ExecuteUpdate", self.updateCallback)
                            end
                            self.updateCount = self.updateCount + 1

                            self:CheckConditions(conditionTableName)
                        end
                    end
                end
            end
        end)
    end

    self.eventCallbackCount = self.eventCallbackCount + 1
    self:CheckConditions(conditionName)
end

function ConditionHandler:RemoveEventCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.eventItemTable) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.eventItemTable[key][key2] = nil
            end
        end
    end

    self.eventCallbackCount = self.eventCallbackCount - 1

    if self.eventCallbackCount == 0 and self.eventTriggerCallbacks then
        RemoveCallback(self, "EventTrigger", self.eventTriggerCallbacks)
        self.eventTriggerCallbacks = nil
    end
end

-- Event Variable handling
function ConditionHandler:AddEventVariableCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do
        local tableName = string.format("EvenVariable|%s", key)
    
        if not self.eventVariableTable[key] then
            self.eventVariableTable[key] = {}
        end
    
        if not self.eventVariableTable[key][conditionName] then
            self.eventVariableTable[key][conditionName] = {}
        end
    
        self.eventVariableTable[key][conditionName][tableName] = 
        {
            ["Value"] = value.Variable_Value and 1 or 0,
            ["Variable"] = value.Variable,
        }
    end

    if self.eventVariableCallbackCount == 0 then
        self.eventVariableTriggerCallbacks = AddCallback(self, "EventTrigger", 
        function(event)
            local value = self.eventVariableTable[event]
            if value then
                for conditionTableName, conditionNameTable in pairs(value) do
                    if conditionNameTable then
                        for tName, eventTableVariable in pairs(conditionNameTable) do
                            self.conditionTables[conditionTableName].StatusTable[eventTableVariable.Variable] = eventTableVariable.Value
                            self:CheckConditions(conditionNameTable)
                        end
                    end
                end
            end
        end)
    end

    self.eventVariableCallbackCount = self.eventVariableCallbackCount + 1
    self:CheckConditions(conditionName)
end

function ConditionHandler:RemoveEventVariableCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.eventVariableTable) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.eventVariableTable[key][key2] = nil
            end
        end
    end

    self.eventVariableCallbackCount = self.eventVariableCallbackCount - 1

    if self.eventVariableCallbackCount == 0 and self.eventVariableTriggerCallbacks then
        RemoveCallback(self, "EventTrigger", self.eventVariableTriggerCallbacks)
        self.eventVariableTriggerCallbacks = nil
    end
end

-- Fight handling
function ConditionHandler:AddFightCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do

        if not self.fightData[value.Boss_Name] then self.fightData[value.Boss_Name] = {} end

        self.fightData[value.Boss_Name][conditionName] = value 
    end


    if self.fightCallbackCount == 0 then
        self.fightEventStartCallbacks = AddCallback(InstanceController, "FightEventStart", function(args)
            if self.fightData[args.Boss_Name] then
                for key, value in pairs(self.fightData[args.Boss_Name]) do
                    if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        self.conditionTables[key].StatusTable[string.format("Fight%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
                        self:CheckConditions(key)
                    end
                end
            end
        end)

        self.fightEventEndCallbacks = AddCallback(InstanceController, "FightEventEnd", function(args)
            if self.fightData[args.Boss_Name] then
                for key, value in pairs(self.fightData[args.Boss_Name]) do
                    if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        self.conditionTables[key].StatusTable[string.format("Fight%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                        self:CheckConditions(key)
                    end
                end
            end
        end)
    end
    self.fightCallbackCount = self.fightCallbackCount + 1
end

function ConditionHandler:RemoveFightCallback(conditionName, conditionType, conditionInfo)

    for key, value in pairs(self.fightData) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.fightData[key][key2] = nil
            end
        end
    end

    self.fightCallbackCount = self.fightCallbackCount - 1

    if self.fightCallbackCount == 0 and self.fightEventStartCallbacks then
        if self.fightEventStartCallbacks then
            RemoveCallback(InstanceController, "FightEventStart", self.fightEventStartCallbacks)
            self.fightEventStartCallbacks = nil
        end

        if self.fightEventEndCallbacks then
            RemoveCallback(InstanceController, "FightEventEnd", self.fightEventEndCallbacks)
            self.fightEventEndCallbacks = nil
        end

    end
end

-- Fight Phase handling
function ConditionHandler:AddFightPhaseCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do

        if not self.fightPhaseData[value.Boss_Name] then self.fightPhaseData[value.Boss_Name] = {} end

        self.fightPhaseData[value.Boss_Name][conditionName] = value 
    end


    if self.fightPhaseCallbackCount == 0 then
        self.fightEventPhaseCallbacks = AddCallback(InstanceController, "FightEventPhase", function(args)
            if self.fightPhaseData[args.Boss_Name] then
                for key, value in pairs(self.fightPhaseData[args.Boss_Name]) do
                    if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        if value.Phase == args.Phase then
                            self.conditionTables[key].StatusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 1
                        else
                            self.conditionTables[key].StatusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                        end
                        self:CheckConditions(key)
                    end
                end
            end
        end)

        self.fightEventEndPhaseCallbacks = AddCallback(InstanceController, "FightEventEnd", function(args)
            if self.fightPhaseData[args.Boss_Name] then
                for key, value in pairs(self.fightPhaseData[args.Boss_Name]) do
                    if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        self.conditionTables[key].StatusTable[string.format("FightPhase%s%s", value.Boss_Name, tostring(value.Tier_Info))] = 0
                        self:CheckConditions(key)
                    end
                end
            end
        end)
    end
    self.fightPhaseCallbackCount = self.fightPhaseCallbackCount + 1
end

function ConditionHandler:RemoveFightPhaseCallback(conditionName, conditionType, conditionInfo)

    for key, value in pairs(self.fightPhaseData) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.fightPhaseData[key][key2] = nil
            end
        end
    end

    self.fightPhaseCallbackCount = self.fightPhaseCallbackCount - 1

    if self.fightPhaseCallbackCount == 0 and self.fightEventPhaseCallbacks then
        if self.fightEventPhaseCallbacks then
            RemoveCallback(InstanceController, "FightEventPhase", self.fightEventPhaseCallbacks)
            self.fightEventPhaseCallbacks = nil
        end

        if self.fightEventEndPhaseCallbacks then
            RemoveCallback(InstanceController, "FightEventEnd", self.fightEventEndPhaseCallbacks)
            self.fightEventEndPhaseCallbacks = nil
        end

    end
end

-- Resource handling
function ConditionHandler:AddResourceCallback(conditionName, conditionType, conditionInfo)

    for key, value in pairs(conditionInfo) do
        local tableName = string.format("Resource|%s", value.Comparator, value.Threshold)
        
        if not self.resourceData[conditionName] then
            self.resourceData[conditionName] = {}
        end
    
        self.resourceData[conditionName][tableName] = 
        {
            ["Comparator"] = value.Comparator or "<",
            ["Threshold"] = value.Threshold or 0,
        }
    end

    if self.attributeCallbackCount == 0 then
        self.ClassAttr = self.player:GetClassAttributes()

        self.attributeFunction = function(number)
            for key, value in pairs(self.resourceData) do
                for key2, value2 in pairs(value) do
                    self.conditionTables[key].StatusTable[key2] = self.compareFunctions[value2.Comparator](number, value2.Threshold) and 1 or 0
                    self:CheckConditions(key)
                end
            end
        end

        if self.playerClass == Turbine.Gameplay.Class.Beorning then
            self.attributeFunction(self.ClassAttr:GetWrath())
            if not(self.attrCallbacks["WrathChanged"] ) then self.attrCallbacks["WrathChanged"] = {} end
            table.insert(self.attrCallbacks["WrathChanged"], AddCallback(self.ClassAttr, "WrathChanged", function(sender, args) 
                self.attributeFunction(self.ClassAttr:GetWrath()) 
            end))
        elseif self.playerClass == Turbine.Gameplay.Class.Champion then
            self.attributeFunction(self.ClassAttr:GetFervor())
            if not(self.attrCallbacks["FervorChanged"] ) then self.attrCallbacks["FervorChanged"] = {} end
            table.insert(self.attrCallbacks["FervorChanged"], AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) 
                self.attributeFunction( self.ClassAttr:GetFervor()) 
            end))
        elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
            self.attributeFunction(self.ClassAttr:GetFocus())
            if not(self.attrCallbacks["FocusChanged"] ) then self.attrCallbacks["FocusChanged"] = {} end
            table.insert(self.attrCallbacks["FocusChanged"], AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) 
                self.attributeFunction( self.ClassAttr:GetFocus()) 
            end))
        elseif self.playerClass == (Turbine.Gameplay.Class.Mariner or Turbine.Gameplay.Class.Corsair or Turbine.Gameplay.Class.Chicken) then
            self.attributeFunction(self.ClassAttr:GetBalance())
            if not(self.attrCallbacks["BalanceChanged"] ) then self.attrCallbacks["BalanceChanged"] = {} end
            table.insert(self.attrCallbacks["BalanceChanged"], AddCallback(self.ClassAttr, "BalanceChanged", function(sender, args) 
                self.attributeFunction( self.ClassAttr:GetBalance()) 
            end))
        elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
            if not(self.attrCallbacks["AttunementChanged"] ) then self.attrCallbacks["AttunementChanged"] = {} end
            self.attributeFunction(self.ClassAttr:GetAttunement())
            table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) 
                self.attributeFunction( self.ClassAttr:GetAttunement()) 
            end))
        end
    end

    self.attributeCallbackCount = self.attributeCallbackCount + 1
end

function ConditionHandler:RemoveResourceCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.resourceData) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.resourceData[key][key2] = nil
            end
        end
    end

    self.attributeCallbackCount = self.attributeCallbackCount - 1

    if  self.attributeCallbackCount == 0 then
        for name, callbackTable in pairs(self.attrCallbacks) do
            for key, callback in pairs(callbackTable) do
                RemoveCallback(self.ClassAttr, name, callback)
                self.attrCallbacks[name][key] = nil
            end
            self.attrCallbacks[name] = nil 
        end
    end
end


-- Class handling
function ConditionHandler:HandleClassConditions(conditionName, conditionType, conditionInfo)
    local getClassName = ClassObjectToNameLUT[self.player:GetClass()]

    if not getClassName then
        ThrowException("Invalid class object passed to conditionhandler")
    end

    if conditionInfo["Class_Match"][getClassName] then
        if not self.conditionTables[conditionName].StatusTable["Class"] then self.conditionTables[conditionName].StatusTable["Class"] = 0 end
        self.conditionTables[conditionName].StatusTable["Class"] = self.conditionTables[conditionName].StatusTable["Class"] + 1
    end
end

-- Race handling
function ConditionHandler:HandleRaceConditions(conditionName, conditionType, conditionInfo)
    local getRaceName = self.raceObjectsToName[self.player:GetRace()]

    if not getRaceName then
        ThrowException("Invalid race object passed to conditionhandler")
    end

    if conditionInfo[getRaceName] then
        if not self.conditionTables[conditionName].StatusTable["Race"] then self.conditionTables[conditionName].StatusTable["Race"] = 0 end
        self.conditionTables[conditionName].StatusTable["Race"] = self.conditionTables[conditionName].StatusTable["Race"] + 1
    end
end

-- Role handling
function ConditionHandler:HandleRoleConditions(conditionName, conditionType, conditionInfo)
    local roleString = Roles:GetRoleText()

    if not roleString then
        ThrowException("Invalid role object passed to conditionhandler")
    end

    if conditionInfo[roleString] then
        self.conditionTables[conditionName].StatusTable["Role"] = 1
    end
end

-- Role handling
function ConditionHandler:HandleLineConditions(conditionName, conditionType, conditionInfo)
    local lineString = Roles:GetLineText()

    if not lineString then
        ThrowException("Invalid line object passed to conditionhandler")
    end

    if conditionInfo[lineString] then
        self.conditionTables[conditionName].StatusTable["Line"] = 1
    end
end

-- Morale handling
function ConditionHandler:AddMoraleCallback(conditionName, conditionType, conditionInfo)
    local moralePercentagePre = tonumber(PlayerHandlerObject:GetInfo().Player_Morale_Percentage)
    for key, value in pairs(conditionInfo) do
        if not self.moraleThresholdTables[key] then self.moraleThresholdTables[key] = {}  end
        local  name = string.format("Morale%s%s%s", value.Threshold or "", value.Comparator or "<", conditionName, key)
        local thresholdValue = self.compareFunctions[value.Comparator or "<"](moralePercentagePre, tonumber(value.Threshold) or 0)
        self.moraleThresholdTables[key][conditionName] = {
            ["Threshold"] = tonumber(value.Threshold) or 0,
            ["Comparator"] = value.Comparator or "<",
            ["Value"] = thresholdValue,
            ["ConditionName"] = conditionName,
            ["Name"] = name,
        }

        self.conditionTables[conditionName].StatusTable[name] = thresholdValue and 1 or 0
    end

    if self.moraleCallbackCount == 0 then
        self.moraleCallback = AddCallback(PlayerHandlerObject, "MoraleChanged", function(args)
            local moralePercentage = tonumber(args.Player_Morale_Percentage)

            for key, value in pairs(self.moraleThresholdTables) do
                for key2, value2 in pairs(value) do
                    local checkValue = self.compareFunctions[value2.Comparator](moralePercentage, value2.Threshold)

                    if checkValue ~= self.moraleThresholdTables[key][key2] then
                        self.moraleThresholdTables[key][key2].Value = checkValue
                        self.conditionTables[value2.ConditionName].StatusTable[self.moraleThresholdTables[key][key2].Name] = checkValue and 1 or 0
                        self:CheckConditions(value2.ConditionName)
                    end
                end
            end
        end)
    end

    self.moraleCallbackCount = self.moraleCallbackCount + 1
end

function ConditionHandler:RemoveMoraleCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.moraleThresholdTables) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.moraleThresholdTables[key][key2] = nil
            end
        end
    end

    self.moraleCallbackCount = self.moraleCallbackCount - 1

    if self.moraleCallbackCount == 0 and self.moraleCallback then
        RemoveCallback(PlayerHandlerObject, "MoraleChanged", self.moraleCallback)
        self.moraleCallback = nil
    end
end

-- Power handling
function ConditionHandler:AddPowerCallback(conditionName, conditionType, conditionInfo)
    local powerPercentagePre = tonumber(PlayerHandlerObject:GetInfo().Player_Power_Percentage)
    for key, value in pairs(conditionInfo) do
        if not self.powerThresholdTables[key] then self.powerThresholdTables[key] = {}  end
        local  name = string.format("Power%s%s%s", value.Threshold or "", value.Comparator or "<", conditionName, key)
        local thresholdValue = self.compareFunctions[value.Comparator or "<"](powerPercentagePre, tonumber(value.Threshold) or 0)
        self.powerThresholdTables[key][conditionName] = {
            ["Threshold"] = tonumber(value.Threshold) or 0,
            ["Comparator"] = value.Comparator or "<",
            ["Value"] = thresholdValue,
            ["ConditionName"] = conditionName,
            ["Name"] = name,
        }

        self.conditionTables[conditionName].StatusTable[name] = thresholdValue and 1 or 0
    end

    if self.powerCallbackCount == 0 then
        self.powerCallback = AddCallback(PlayerHandlerObject, "PowerChanged", function(args)
            local powerPercentage = tonumber(args.Player_Power_Percentage)

            for key, value in pairs(self.powerThresholdTables) do
                for key2, value2 in pairs(value) do
                    local checkValue = self.compareFunctions[value2.Comparator](powerPercentage, value2.Threshold)

                    if checkValue ~= self.powerThresholdTables[key][key2] then
                        self.powerThresholdTables[key][key2].Value = checkValue
                        self.conditionTables[value2.ConditionName].StatusTable[self.powerThresholdTables[key][key2].Name] = checkValue and 1 or 0
                        self:CheckConditions(value2.ConditionName)
                    end
                end
            end
        end)
    end

    self.powerCallbackCount = self.powerCallbackCount + 1
end

function ConditionHandler:RemovePowerCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.powerThresholdTables) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.powerThresholdTables[key][key2] = nil
            end
        end
    end

    self.powerCallbackCount = self.powerCallbackCount - 1

    if self.powerCallbackCount == 0 and self.powerCallback then
        RemoveCallback(PlayerHandlerObject, "PowerChanged", self.powerCallback)
        self.powerCallback = nil
    end
end

-- Level handling
function ConditionHandler:AddLevelCallback(conditionName, conditionType, conditionInfo)
    local level = tonumber(PlayerHandlerObject:GetInfo().Player_Level)
    for key, value in pairs(conditionInfo) do
        if not self.levelThresholdTables[key] then self.levelThresholdTables[key] = {}  end
        local  name = string.format("Level%s%s%s", value.Threshold or "", value.Comparator or "<", conditionName, key)
        local thresholdValue = self.compareFunctions[value.Comparator or "<"](level, tonumber(value.Threshold) or 0)
        self.levelThresholdTables[key][conditionName] = {
            ["Threshold"] = tonumber(value.Threshold) or 0,
            ["Comparator"] = value.Comparator or "<",
            ["Value"] = thresholdValue,
            ["ConditionName"] = conditionName,
            ["Name"] = name,
        }

        self.conditionTables[conditionName].StatusTable[name] = thresholdValue and 1 or 0
    end

    if self.levelCallbackCount == 0 then
        self.levelCallback = AddCallback(PlayerHandlerObject, "LevelChanged", function(args)
            local level = tonumber(args.Player_Level)

            for key, value in pairs(self.levelThresholdTables) do
                for key2, value2 in pairs(value) do
                    local checkValue = self.compareFunctions[value2.Comparator](level, value2.Threshold)

                    if checkValue ~= self.levelThresholdTables[key][key2] then
                        self.levelThresholdTables[key][key2].Value = checkValue
                        self.conditionTables[value2.ConditionName].StatusTable[self.levelThresholdTables[key][key2].Name] = checkValue and 1 or 0
                        self:CheckConditions(value2.ConditionName)
                    end
                end
            end
        end)
    end

    self.levelCallbackCount = self.levelCallbackCount + 1
end

function ConditionHandler:RemoveLevelCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.levelThresholdTables) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.levelThresholdTables[key][key2] = nil
            end
        end
    end

    self.levelCallbackCount = self.levelCallbackCount - 1

    if self.levelCallbackCount == 0 and self.levelCallback then
        RemoveCallback(PlayerHandlerObject, "LevelChanged", self.levelCallback)
        self.levelCallback = nil
    end
end

-- Loot handling
function ConditionHandler:AddLootCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do

        if not self.lockTable[key] then self.lockTable[key] = {} end

        self.lockTable[key][conditionName] = value.Not_Applied

        local lootValue = LootHandler:GetLockStatus(key)

        if not lootValue then
            lootValue = false
        else
            lootValue = (tonumber(lootValue) or 0) <= 0
        end

        --true is used | false is available or not detected

        self.conditionTables[conditionName].StatusTable[string.format("Lock%s", key)] = lootValue and value.Not_Applied and 1 or not lootValue and not value.Not_Applied and 1 or 0
    end

    if self.lockCallbackCount == 0 then
        self.lootCallback = AddCallback(LootHandler, "LootLockChanged", function(args)
            if self.lockTable[args.Token] then
                local lootValue = LootHandler:GetLockStatus(args.Token)
                if not lootValue then
                    lootValue = false
                else
                    lootValue = (tonumber(lootValue) or 0) <= 0
                end



                for key, value in pairs(self.lockTable[args.Token]) do
                    self.conditionTables[key].StatusTable[string.format("Lock%s", args.Token)] = lootValue and value and 1 or not lootValue and not value and 1 or 0
                    self:CheckConditions(key)
                end
            end
        end)
    end

    self.lockCallbackCount = self.lockCallbackCount + 1
end

function ConditionHandler:RemoveLootCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.lockTable) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.lockTable[key][key2] = nil
            end
        end
    end

    self.lockCallbackCount = self.lockCallbackCount - 1

    if self.lockCallbackCount == 0 and self.lootCallback then
        RemoveCallback(LootHandler, "LootLockChanged", self.lootCallback)
        self.lootCallback = nil
    end
end

-- Quest handling
function ConditionHandler:AddQuestCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(conditionInfo) do

        if not self.questTable[key] then self.questTable[key] = {} end

        self.questTable[key][conditionName] = value.Not_Applied

        local lootValue = QuestHandlerObject:IsAcceptedQuest(key)

        --true is used | false is available or not detected

        self.conditionTables[conditionName].StatusTable[string.format("Quest%s", key)] = lootValue and value.Not_Applied and 1 or not lootValue and not value.Not_Applied and 1 or 0
    end

    if self.questCallbackCount == 0 then
        self.acceptedCallback = AddCallback(QuestHandlerObject, "QuestAccepted", function(args)
            if self.questTable[args.Name] then

                for key, value in pairs(self.questTable[args.Name]) do
                    self.conditionTables[key].StatusTable[string.format("Quest%s", args.Name)] = not value and 1 or 0
                    self:CheckConditions(key)
                end
            end
        end)

        self.failedCallback = AddCallback(QuestHandlerObject, "QuestFailed", function(args)
            if self.questTable[args.Name] then

                for key, value in pairs(self.questTable[args.Name]) do
                    self.conditionTables[key].StatusTable[string.format("Quest%s", args.Name)] =  value and 1 or 0
                    self:CheckConditions(key)
                end
            end
        end)

        self.completedCallback = AddCallback(QuestHandlerObject, "QuestCompleted", function(args)
            if self.questTable[args.Name] then

                for key, value in pairs(self.questTable[args.Name]) do
                    self.conditionTables[key].StatusTable[string.format("Quest%s", args.Name)] = value and 1 or 0
                    self:CheckConditions(key)
                end
            end
        end)
    end

    self.questCallbackCount = self.questCallbackCount + 1
end

function ConditionHandler:RemoveQuestCallback(conditionName, conditionType, conditionInfo)
    for key, value in pairs(self.questTable) do
        for key2, value2 in pairs(value) do
            if key2 == conditionName then
                self.questTable[key][key2] = nil
            end
        end
    end

    self.questCallbackCount = self.questCallbackCount - 1

    if self.questCallbackCount == 0 then
        if self.acceptedCallback then
            RemoveCallback(QuestHandlerObject, "QuestAccepted", self.acceptedCallback)
            self.acceptedCallback = nil
        end

        if self.failedCallback then
            RemoveCallback(QuestHandlerObject, "QuestFailed", self.failedCallback)
            self.failedCallback = nil
        end

        if self.completedCallback then
            RemoveCallback(QuestHandlerObject, "QuestCompleted", self.completedCallback)
            self.completedCallback = nil
        end
    end
end

--[[

{
        ["Name"] = "Test",
        ["Threshold"] = 1,
        ["Conditions"] = {
            ["In_Combat"] = true,
            ["Effects"] = {
                ["Swordplay: En Garde"] = {
                    ["Applications"] = 2,
                    ["Not_Applied"] = false,
                }
            },
            ["Skill_Ready"] = {
                ["Step Back"] = {
                    ["Not_Applied"] = true,
                }
            },
            ["Mounted"] = {
                {["Name"] = "Mounted"},
            },
            ["Class"] = {
                ["Class_Match"] = {["LoreMaster"] = false, ["Mariner"] = false },
            },
            ["Role"] = {
                ["Support"] = false,
            },
            ["Line"] = {
                ["Yellow"] = false,
            },
            ["Location"] = {
                ["Shire"] = true,
            },
            ["Power"] = {
                {
                    ["Threshold"] = "99",
                    ["Comparator"] = "<",
                },
            },
            ["EventTrigger"] = {
                ["TestEvent"] = {
                    ["Not_Applied"] = true,
                    ["Timer"] = 10,
                },
                ["TestEvent2"] = {
                    ["Not_Applied"] = false,
                    ["Timer"] = true,
                },
            },
            ["Fight"] = {
                ["Rakothas"] = {
                    ["Tier_Info"] = true,
                    ["Boss_Name"] = "Rakothas",
                }
            },
            ["Status"] = {
                ["Wound"] = true,
                ["Fear"] = true,
            },
            ["EventVariable"] = {
                ["VariableTest"] = {
                    ["Variable_Value"] = false,
                    ["Variable"] = "Test1",
                },
                ["VariableTest2"] = {
                    ["Variable_Value"] = true,
                    ["Variable"] = "Test1",
                },
            },
            ["Resource"] = {
                {
                    ["Comparator"] = ">",
                    ["Threshold"] = 25,
                }
            }
        }

    }

        --ConditionHandler:FireTrigger("TestEvent2", {["Timer"] = 10})
    ConditionHandler:FireTrigger("VariableTest", {})
    ConditionHandler:FireTrigger("VariableTest2", {})
    ConditionHandler:FireTrigger("VariableTest", {})
    --ConditionHandler:FireTrigger("VariableTest2", {})
]]