QuestHandler = class()
function QuestHandler:Constructor()
    self.questsAccepted = LoadTable(Turbine.DataScope.Character, "PrimeQuestsAccepted") or {}
    self.questsCompleted = LoadTable(Turbine.DataScope.Character, "PrimeQuestsCompleted") or {}

    self.Event = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    self:SetCallbacks()
end

function QuestHandler:GetAllAcceptedQuests()
    return self.questsAccepted
end

function QuestHandler:GetAllCompletedQuests()
    return self.questsCompleted
end

function QuestHandler:IsAcceptedQuest(name)
    return self.questsAccepted[name] ~= nil
end

function QuestHandler:IsCompletedQuest(name)
    return self.questsCompleted[name] and self.questsCompleted[name].Completed 
end

function QuestHandler:RemoveQuest(name)
    self.questsAccepted[name] = nil
end

function QuestHandler:SetCallbacks()
    if not self.questAcceptedCallback then
        self.questAcceptedCallback = AddCallback(ChatEventHandler, "QuestAccepted", function(args)
            self.questsAccepted[args.Name] = true
            self.Event(self, self.QuestAccepted, {["Name"] = args.Name, ["Completed_Before"] = self.questsCompleted[args.Name] and self.questsCompleted[args.Name].Completed > 0})
        end)
    end

    if not self.questFailedCallback then
        self.questFailedCallback = AddCallback(ChatEventHandler, "QuestFailed", function(args)
            self.questsAccepted[args.Name] = nil
            self.Event(self, self.QuestFailed, {["Name"] = args.Name, ["Completed_Before"] = self.questsCompleted[args.Name] and self.questsCompleted[args.Name].Completed > 0})
        end)
    end

    if not self.questCompletedCallback then
        self.questCompletedCallback = AddCallback(ChatEventHandler, "QuestOrDeedCompleted", function(args)
            if not self.questsCompleted[args.Name] then self.questsCompleted[args.Name] = {["Completed"] = 0} end
            self.questsCompleted[args.Name].Completed = self.questsCompleted[args.Name].Completed + 1
            self:RemoveQuest(args.Name)
            self.Event(self, self.QuestCompleted, {["Name"] = args.Name, ["Completed_Before"] = self.questsCompleted[args.Name] and self.questsCompleted[args.Name].Completed > 0})
        end)
    end
end

function QuestHandler:RemoveCallbacks()
    if self.questAcceptedCallback then
        RemoveCallback(ChatEventHandler, "QuestAccepted", self.questAcceptedCallback)
        self.questAcceptedCallback = nil
    end

    if self.questFailedCallback then
        RemoveCallback(ChatEventHandler, "QuestFailed", self.questFailedCallback)
        self.questFailedCallback = nil
    end

    if self.questCompletedCallback then
        RemoveCallback(ChatEventHandler, "QuestOrDeedCompleted", self.questCompletedCallback)
        self.questCompletedCallback = nil
    end
end

function QuestHandler:Unload()
    SaveTable(Turbine.DataScope.Character, "PrimeQuestsAccepted", self.questsAccepted)
    SaveTable(Turbine.DataScope.Character, "PrimeQuestsCompleted", self.questsCompleted)
end
