ExperienceHandler = class()
function ExperienceHandler:Constructor()
    self.levelTable = GetLevelTable()

    self.LevelEvent = function(sender, event, args)
        if type(event)=="function" then
            event(args)
        else
            if type(event)=="table" then
                for i=1,#event do
                    if type(event[i])=="function" then
                        event[i](args)
                    end
                end
            end
        end
    end

    local loadData = LoadTable(Turbine.DataScope.Character, "PrimeExperienceData")
    local timeOnLevelData = LoadTable(Turbine.DataScope.Character, "PrimeTimeOnLevelData")

    self.player = Turbine.Gameplay.LocalPlayer.GetInstance()
    self.currentLevel = self.player:GetLevel()

    if loadData then
        self.percentage = loadData.Experience_Progress
        self.total = loadData.Experience_Total
        self.total_Needed = loadData.Experience_Total_Needed
        self.relativeCurrent = loadData.Experience_Relative_Current
        self.relativeNeeded = loadData.Experience_Relative_Needed
    else
        self.percentage = 0
        self.total = 0
        self.total_Needed = 0
        self.relativeCurrent = 0
        self.relativeNeeded = 0
    end

    if timeOnLevelData then
        self.timeOnLevel = timeOnLevelData
    else
        self.timeOnLevel = {}
    end

    self.lastBroadcast = Turbine.Engine.GetGameTime() - 60

    self:SetCallbacks()
end

function ExperienceHandler:SetCallbacks()
    -- Add Level Changed Callback
    --self.UpdateEvent(self, self.LevelChanged, level)

    -- Add Chat Callback
    --self.UpdateEvent(self, self.ExperienceChanged, args)
    self.chatCallback = AddCallback(ChatEventHandler, "Experience", function(args)

        local levelInfoCur = self.levelTable[self.player:GetLevel()]
        local levelInfoUp = self.levelTable[self.player:GetLevel() + 1]
        if levelInfoCur and levelInfoUp then
            local relativeCurrent = args.Total - levelInfoCur.Total_Required
            local percentage = ((relativeCurrent) / (levelInfoUp.Total_Required - levelInfoCur.Total_Required)) * 100

            self.percentage = percentage
            self.total = args.Total
            self.total_Needed =  levelInfoUp.Total_Required
            self.relativeCurrent = relativeCurrent
            self.relativeNeeded = levelInfoUp.Relative_Required

            self.LevelEvent(self, self.ExperienceUpdate, {
                ["Experience_Progress"] = self.percentage, 
                ["Experience_Total"] = self.total, 
                ["Experience_Total_Needed"] = self.total_Needed, 
                ["Experience_Relative_Current"] = self.relativeCurrent,
                ["Experience_Relative_Needed"] = self.relativeNeeded})
        end
    end)

    self.updateCallback = AddCallback(Updater, "ExecuteUpdate", function(delta) 
        self.timeOnLevel[self.player:GetLevel()] = (self.timeOnLevel[self.player:GetLevel()] or 0) + delta

        if self.lastBroadcast + 1 < Turbine.Engine.GetGameTime() then
            self.lastBroadcast = Turbine.Engine.GetGameTime()

            self.LevelEvent(self, self.ExperienceLevelUpdate, {
                ["Experience_Time_On_Level"] = self.timeOnLevel[self.player:GetLevel()] or 0,})
        end
    end)
end

function ExperienceHandler:GetCurrentInfo()
    return {                
        ["Experience_Progress"] = self.percentage, 
        ["Experience_Total"] = self.total, 
        ["Experience_Total_Needed"] = self.total_Needed, 
        ["Experience_Relative_Current"] = self.relativeCurrent,
        ["Experience_Relative_Needed"] = self.relativeNeeded,
    }
end

function ExperienceHandler:GetCurrentTimeInfo()
    return self.timeOnLevel[self.player:GetLevel()] or 0
end

function ExperienceHandler:RemoveCallbacks()
    -- Remove Level Changed Callback
    -- Remove Chat Callback

    RemoveCallback(ChatEventHandler, "Experience", self.chatCallback)
    RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback)
end

function ExperienceHandler:Unload()
    self:RemoveCallbacks()

    SaveTable(Turbine.DataScope.Character, "PrimeExperienceData", 
    {
        ["Experience_Progress"] = self.percentage, 
        ["Experience_Total"] = self.total, 
        ["Experience_Total_Needed"] = self.total_Needed, 
        ["Experience_Relative_Current"] = self.relativeCurrent,
        ["Experience_Relative_Needed"] = self.relativeNeeded,
    })

    SaveTable(Turbine.DataScope.Character, "PrimeTimeOnLevelData", self.timeOnLevel)
end

function GetLevelTable()
    
    return
    { 
        { ["level"] = 1, ["Relative_Required"] = 0, ["Total_Required"] = 0 },
        { ["level"] = 2, ["Relative_Required"] = 100, ["Total_Required"] = 100 },
        { ["level"] = 3, ["Relative_Required"] = 175, ["Total_Required"] = 275 },
        { ["level"] = 4, ["Relative_Required"] = 275, ["Total_Required"] = 550 },
        { ["level"] = 5, ["Relative_Required"] = 400, ["Total_Required"] = 950 },
        { ["level"] = 6, ["Relative_Required"] = 593, ["Total_Required"] = 1543 },
        { ["level"] = 7, ["Relative_Required"] = 852, ["Total_Required"] = 2395 },
        { ["level"] = 8, ["Relative_Required"] = 1180, ["Total_Required"] = 3575 },
        { ["level"] = 9, ["Relative_Required"] = 1575, ["Total_Required"] = 5150 },
        { ["level"] = 10, ["Relative_Required"] = 2038, ["Total_Required"] = 7188 },
        { ["level"] = 11, ["Relative_Required"] = 2610, ["Total_Required"] = 9798 },
        { ["level"] = 12, ["Relative_Required"] = 3292, ["Total_Required"] = 13090 },
        { ["level"] = 13, ["Relative_Required"] = 4085, ["Total_Required"] = 17175 },
        { ["level"] = 14, ["Relative_Required"] = 4988, ["Total_Required"] = 22163 },
        { ["level"] = 15, ["Relative_Required"] = 6000, ["Total_Required"] = 28163 },
        { ["level"] = 16, ["Relative_Required"] = 7165, ["Total_Required"] = 35328 },
        { ["level"] = 17, ["Relative_Required"] = 8482, ["Total_Required"] = 43810 },
        { ["level"] = 18, ["Relative_Required"] = 9953, ["Total_Required"] = 53763 },
        { ["level"] = 19, ["Relative_Required"] = 11575, ["Total_Required"] = 65338 },
        { ["level"] = 20, ["Relative_Required"] = 13350, ["Total_Required"] = 78688 },
        { ["level"] = 21, ["Relative_Required"] = 15320, ["Total_Required"] = 94008 },
        { ["level"] = 22, ["Relative_Required"] = 17485, ["Total_Required"] = 111493 },
        { ["level"] = 23, ["Relative_Required"] = 19845, ["Total_Required"] = 131338 },
        { ["level"] = 24, ["Relative_Required"] = 22400, ["Total_Required"] = 153738 },
        { ["level"] = 25, ["Relative_Required"] = 25150, ["Total_Required"] = 178888 },
        { ["level"] = 26, ["Relative_Required"] = 28137, ["Total_Required"] = 207025 },
        { ["level"] = 27, ["Relative_Required"] = 31363, ["Total_Required"] = 238388 },
        { ["level"] = 28, ["Relative_Required"] = 34825, ["Total_Required"] = 273213 },
        { ["level"] = 29, ["Relative_Required"] = 38525, ["Total_Required"] = 311738 },
        { ["level"] = 30, ["Relative_Required"] = 42462, ["Total_Required"] = 354200 },
        { ["level"] = 31, ["Relative_Required"] = 46680, ["Total_Required"] = 400880 },
        { ["level"] = 32, ["Relative_Required"] = 51178, ["Total_Required"] = 452058 },
        { ["level"] = 33, ["Relative_Required"] = 55955, ["Total_Required"] = 508013 },
        { ["level"] = 34, ["Relative_Required"] = 61012, ["Total_Required"] = 569025 },
        { ["level"] = 35, ["Relative_Required"] = 66350, ["Total_Required"] = 635375 },
        { ["level"] = 36, ["Relative_Required"] = 72010, ["Total_Required"] = 707385 },
        { ["level"] = 37, ["Relative_Required"] = 77993, ["Total_Required"] = 785378 },
        { ["level"] = 38, ["Relative_Required"] = 84297, ["Total_Required"] = 869675 },
        { ["level"] = 39, ["Relative_Required"] = 90925, ["Total_Required"] = 960600 },
        { ["level"] = 40, ["Relative_Required"] = 97875, ["Total_Required"] = 1058475 },
        { ["level"] = 41, ["Relative_Required"] = 105190, ["Total_Required"] = 1163665 },
        { ["level"] = 42, ["Relative_Required"] = 112870, ["Total_Required"] = 1276535 },
        { ["level"] = 43, ["Relative_Required"] = 120915, ["Total_Required"] = 1397450 },
        { ["level"] = 44, ["Relative_Required"] = 129325, ["Total_Required"] = 1526775 },
        { ["level"] = 45, ["Relative_Required"] = 138100, ["Total_Required"] = 1664875 },
        { ["level"] = 46, ["Relative_Required"] = 147283, ["Total_Required"] = 1812158 },
        { ["level"] = 47, ["Relative_Required"] = 156872, ["Total_Required"] = 1969030 },
        { ["level"] = 48, ["Relative_Required"] = 166870, ["Total_Required"] = 2135900 },
        { ["level"] = 49, ["Relative_Required"] = 177275, ["Total_Required"] = 2313175 },
        { ["level"] = 50, ["Relative_Required"] = 188088, ["Total_Required"] = 2501263 },
        { ["level"] = 51, ["Relative_Required"] = 199350, ["Total_Required"] = 2700613 },
        { ["level"] = 52, ["Relative_Required"] = 211062, ["Total_Required"] = 2911675 },
        { ["level"] = 53, ["Relative_Required"] = 223225, ["Total_Required"] = 3134900 },
        { ["level"] = 54, ["Relative_Required"] = 235838, ["Total_Required"] = 3370738 },
        { ["level"] = 55, ["Relative_Required"] = 248900, ["Total_Required"] = 3619638 },
        { ["level"] = 56, ["Relative_Required"] = 262455, ["Total_Required"] = 3882093 },
        { ["level"] = 57, ["Relative_Required"] = 276502, ["Total_Required"] = 4158595 },
        { ["level"] = 58, ["Relative_Required"] = 291043, ["Total_Required"] = 4449638 },
        { ["level"] = 59, ["Relative_Required"] = 306075, ["Total_Required"] = 4755713 },
        { ["level"] = 60, ["Relative_Required"] = 321600, ["Total_Required"] = 5077313 },
        { ["level"] = 61, ["Relative_Required"] = 337912, ["Total_Required"] = 5415225 },
        { ["level"] = 62, ["Relative_Required"] = 355051, ["Total_Required"] = 5770276 },
        { ["level"] = 63, ["Relative_Required"] = 373059, ["Total_Required"] = 6143335 },
        { ["level"] = 64, ["Relative_Required"] = 391980, ["Total_Required"] = 6535315 },
        { ["level"] = 65, ["Relative_Required"] = 411861, ["Total_Required"] = 6947176 },
        { ["level"] = 66, ["Relative_Required"] = 432750, ["Total_Required"] = 7379926 },
        { ["level"] = 67, ["Relative_Required"] = 454698, ["Total_Required"] = 7834624 },
        { ["level"] = 68, ["Relative_Required"] = 477759, ["Total_Required"] = 8312383 },
        { ["level"] = 69, ["Relative_Required"] = 501990, ["Total_Required"] = 8814373 },
        { ["level"] = 70, ["Relative_Required"] = 527450, ["Total_Required"] = 9341823 },
        { ["level"] = 71, ["Relative_Required"] = 554201, ["Total_Required"] = 9896024 },
        { ["level"] = 72, ["Relative_Required"] = 582309, ["Total_Required"] = 10478333 },
        { ["level"] = 73, ["Relative_Required"] = 611843, ["Total_Required"] = 11090176 },
        { ["level"] = 74, ["Relative_Required"] = 642875, ["Total_Required"] = 11733051 },
        { ["level"] = 75, ["Relative_Required"] = 675481, ["Total_Required"] = 12408532 },
        { ["level"] = 76, ["Relative_Required"] = 709255, ["Total_Required"] = 13117787 },
        { ["level"] = 77, ["Relative_Required"] = 744717, ["Total_Required"] = 13862504 },
        { ["level"] = 78, ["Relative_Required"] = 781952, ["Total_Required"] = 14644456 },
        { ["level"] = 79, ["Relative_Required"] = 821049, ["Total_Required"] = 15465505 },
        { ["level"] = 80, ["Relative_Required"] = 862101, ["Total_Required"] = 16327606 },
        { ["level"] = 81, ["Relative_Required"] = 905206, ["Total_Required"] = 17232812 },
        { ["level"] = 82, ["Relative_Required"] = 950466, ["Total_Required"] = 18183278 },
        { ["level"] = 83, ["Relative_Required"] = 997989, ["Total_Required"] = 19181267 },
        { ["level"] = 84, ["Relative_Required"] = 1047888, ["Total_Required"] = 20229155 },
        { ["level"] = 85, ["Relative_Required"] = 1100282, ["Total_Required"] = 21329437 },
        { ["level"] = 86, ["Relative_Required"] = 1155296, ["Total_Required"] = 22484733 },
        { ["level"] = 87, ["Relative_Required"] = 1213060, ["Total_Required"] = 23697793 },
        { ["level"] = 88, ["Relative_Required"] = 1273713, ["Total_Required"] = 24971506 },
        { ["level"] = 89, ["Relative_Required"] = 1337398, ["Total_Required"] = 26308904 },
        { ["level"] = 90, ["Relative_Required"] = 1404267, ["Total_Required"] = 27713171 },
        { ["level"] = 91, ["Relative_Required"] = 1474480, ["Total_Required"] = 29187651 },
        { ["level"] = 92, ["Relative_Required"] = 1548204, ["Total_Required"] = 30735855 },
        { ["level"] = 93, ["Relative_Required"] = 1625614, ["Total_Required"] = 32361469 },
        { ["level"] = 94, ["Relative_Required"] = 1706894, ["Total_Required"] = 34068363 },
        { ["level"] = 95, ["Relative_Required"] = 1792238, ["Total_Required"] = 35860601 },
        { ["level"] = 96, ["Relative_Required"] = 1881849, ["Total_Required"] = 37742450 },
        { ["level"] = 97, ["Relative_Required"] = 1975941, ["Total_Required"] = 39718391 },
        { ["level"] = 98, ["Relative_Required"] = 2074738, ["Total_Required"] = 41793129 },
        { ["level"] = 99, ["Relative_Required"] = 2178474, ["Total_Required"] = 43971603 },
        { ["level"] = 100, ["Relative_Required"] = 2287397, ["Total_Required"] = 46259000 },
        { ["level"] = 101, ["Relative_Required"] = 2401766, ["Total_Required"] = 48660766 },
        { ["level"] = 102, ["Relative_Required"] = 2521854, ["Total_Required"] = 51182620 },
        { ["level"] = 103, ["Relative_Required"] = 2647946, ["Total_Required"] = 53830566 },
        { ["level"] = 104, ["Relative_Required"] = 2780343, ["Total_Required"] = 56610909 },
        { ["level"] = 105, ["Relative_Required"] = 2919360, ["Total_Required"] = 59530269 },
        { ["level"] = 106, ["Relative_Required"] = 3065328, ["Total_Required"] = 62595597 },
        { ["level"] = 107, ["Relative_Required"] = 3218594, ["Total_Required"] = 65814191 },
        { ["level"] = 108, ["Relative_Required"] = 3379523, ["Total_Required"] = 69193714 },
        { ["level"] = 109, ["Relative_Required"] = 3548499, ["Total_Required"] = 72742213 },
        { ["level"] = 110, ["Relative_Required"] = 3725923, ["Total_Required"] = 76468136 },
        { ["level"] = 111, ["Relative_Required"] = 3912219, ["Total_Required"] = 80380355 },
        { ["level"] = 112, ["Relative_Required"] = 4107829, ["Total_Required"] = 84488184 },
        { ["level"] = 113, ["Relative_Required"] = 4313220, ["Total_Required"] = 88801404 },
        { ["level"] = 114, ["Relative_Required"] = 4528881, ["Total_Required"] = 93330285 },
        { ["level"] = 115, ["Relative_Required"] = 4755325, ["Total_Required"] = 98085610 },
        { ["level"] = 116, ["Relative_Required"] = 4993091, ["Total_Required"] = 103078701 },
        { ["level"] = 117, ["Relative_Required"] = 5242745, ["Total_Required"] = 108321446 },
        { ["level"] = 118, ["Relative_Required"] = 5504882, ["Total_Required"] = 113826328 },
        { ["level"] = 119, ["Relative_Required"] = 5780126, ["Total_Required"] = 119606454 },
        { ["level"] = 120, ["Relative_Required"] = 6069132, ["Total_Required"] = 125675586 },
        { ["level"] = 121, ["Relative_Required"] = 6372588, ["Total_Required"] = 132048174 },
        { ["level"] = 122, ["Relative_Required"] = 6691217, ["Total_Required"] = 138739391 },
        { ["level"] = 123, ["Relative_Required"] = 7025777, ["Total_Required"] = 145765168 },
        { ["level"] = 124, ["Relative_Required"] = 7377065, ["Total_Required"] = 153142233 },
        { ["level"] = 125, ["Relative_Required"] = 7745918, ["Total_Required"] = 160888151 },
        { ["level"] = 126, ["Relative_Required"] = 8133213, ["Total_Required"] = 169021364 },
        { ["level"] = 127, ["Relative_Required"] = 8539873, ["Total_Required"] = 177561237 },
        { ["level"] = 128, ["Relative_Required"] = 8966866, ["Total_Required"] = 186528103 },
        { ["level"] = 129, ["Relative_Required"] = 9415209, ["Total_Required"] = 195943312 },
        { ["level"] = 130, ["Relative_Required"] = 9885969, ["Total_Required"] = 205829281 },
        { ["level"] = 131, ["Relative_Required"] = 10380267, ["Total_Required"] = 216209548 },
        { ["level"] = 132, ["Relative_Required"] = 10899280, ["Total_Required"] = 227108828 },
        { ["level"] = 133, ["Relative_Required"] = 11444244, ["Total_Required"] = 238553072 },
        { ["level"] = 134, ["Relative_Required"] = 12016456, ["Total_Required"] = 250569528 },
        { ["level"] = 135, ["Relative_Required"] = 12617278, ["Total_Required"] = 263186806 },
        { ["level"] = 136, ["Relative_Required"] = 13248141, ["Total_Required"] = 276434947 },
        { ["level"] = 137, ["Relative_Required"] = 13910548, ["Total_Required"] = 290345495 },
        { ["level"] = 138, ["Relative_Required"] = 14606075, ["Total_Required"] = 304951570 },
        { ["level"] = 139, ["Relative_Required"] = 15336378, ["Total_Required"] = 320287948 },
        { ["level"] = 140, ["Relative_Required"] = 16103196, ["Total_Required"] = 336391144 },
        { ["level"] = 141, ["Relative_Required"] = 16908355, ["Total_Required"] = 353299499 },
        { ["level"] = 142, ["Relative_Required"] = 17753772, ["Total_Required"] = 371053271 },
        { ["level"] = 143, ["Relative_Required"] = 18641460, ["Total_Required"] = 389694731 },
        { ["level"] = 144, ["Relative_Required"] = 19573533, ["Total_Required"] = 409268264 },
        { ["level"] = 145, ["Relative_Required"] = 20552209, ["Total_Required"] = 429820473 },
        { ["level"] = 146, ["Relative_Required"] = 21579819, ["Total_Required"] = 451400292 },
        { ["level"] = 147, ["Relative_Required"] = 22658809, ["Total_Required"] = 474059101 },
        { ["level"] = 148, ["Relative_Required"] = 23791749, ["Total_Required"] = 497850850 },
        { ["level"] = 149, ["Relative_Required"] = 24981336, ["Total_Required"] = 522832186 },
        { ["level"] = 150, ["Relative_Required"] = 26230402, ["Total_Required"] = 549062588 },
    }
end