PrimeUIFigure = class(Turbine.UI.Window)
function PrimeUIFigure:Constructor(data, parent, colour)
    Turbine.UI.Window.Constructor(self)

    self.data = data
    self.data.Toggle_Type = "Flash"

    self.duration = 0
    self.maxDuration = self.data.Max_Duration
    self.flashDuration = self.data.Flash_Duration
    self.maxOpacity = self.data.Max_Opacity
    self.flashOpacity = self.data.Flash_Opacity
    self.processOpacity = (self.maxOpacity - self.flashOpacity) / self.flashDuration
    if self.processOpacity == 0 then self.processOpacity = self.maxOpacity / self.maxDuration * 2 end
    self.down = true

    if self.data.Use_Screen_Width then
        self.data.Width = Turbine.UI.Display.GetWidth()
    end

    if self.data.Use_Screen_Height then
        self.data.Height = Turbine.UI.Display.GetHeight()
    end

    self:SetParent(parent)
    self:SetPosition(self.data.X, self.data.Y)
    self:SetVisible(self.data.Visible_On_Load)
    self:SetMouseVisible(false)

    if self.data.Use_Texture then
        self:SetSize(self.data.Width, self.data.Height)
        self.textureWindow = Turbine.UI.Window()
        self.textureWindow:SetParent(self)
        self.textureWindow:SetVisible(true)
        self.textureWindow:SetMouseVisible(false)
        self.textureWindow:SetBackground(self.data.Texture)
        if self.data.Use_Texture_Colour then
            self.textureWindow:SetBackColor(ColourTupleToTurbineColour(self.data.Texture_Colour))
            self.textureWindow:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
        end

        self.textureWindow:SetStretchMode(2)
        --self.textureWindow:SetSize(self.textureWindow:GetWidth(), self.textureWindow:GetHeight())
        self.textureWindow:SetStretchMode(1)
        self.textureWindow:SetSize(self.data.Width, self.data.Height)
        self.scaledWidth = self.data.Width * 0.5
        self.scaledHeight = self.data.Height * 0.5
    else
        self:SetBackColor(colour)
        self.scaledWidth = self:GetWidth() * 0.5
        self.scaledHeight = self:GetHeight() * 0.5
        self:SetSize(self.data.Width, self.data.Height)
    end

    if self.data.useColour then
        self:SetBackColor(colour)
        self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend)
    end

    self:SetOpacity(self.flashOpacity)

    self.ExecuteUpdate = function(delta)
        if self.duration > 0 then
            self.duration = self.duration - delta
            if self.down then
                self:SetOpacity(self:GetOpacity() - self.processOpacity * delta)
                if self:GetOpacity() <= 0 then
                    self:SetVisible(false)
                    if self.textureWindow then self.textureWindow:SetVisible(false) end
                    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
                    self.updateCallback = nil
                end
            else
                self:SetOpacity(math.min(self.maxOpacity, self:GetOpacity() + self.processOpacity * delta))
                if self:GetOpacity() + 0.05 >= self.maxOpacity then
                    self.down = true
                end
            end
        else
            if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
            self.updateCallback = nil
        end
    end

    if self.data.Use_Mouse_As_Position then
        self.ExecuteUpdateMouse = function(delta)
            local mouseX, mouseY = Turbine.UI.Display:GetMousePosition()
            self:SetPosition(mouseX - self.scaledWidth + self.data.Mouse_Offset_X, mouseY - self.scaledHeight + self.data.Mouse_Offset_X)
        end
        self.mouseCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdateMouse)
    end



    --self:Toggle()
end

function PrimeUIFigure:Toggle()

    if self.data.Toggle_Type == "Flash" then
        self.duration = self.maxDuration
        self.flashes = 0
        self.down = false
        self:SetVisible(true)
        if self.textureWindow then self.textureWindow:SetVisible(true) end
        self:SetOpacity(self.flashOpacity)
        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end

    elseif self.data.Toggle_Type == "Show" then

    end
end

function PrimeUIFigure:Unload()
    if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end
    self.updateCallback = nil

    if self.mouseCallback then RemoveCallback(Updater, "ExecuteUpdate", self.mouseCallback) end
    self.mouseCallback = nil
end