function PrimeUI:PartyEvent(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    self.lookup[data.NickName or ""] = valuebar
    valuebar.Active = true

    if tablelength(data["Start_Keyword"]) > 0 then
        valuebar.Active = false
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Start_Keyword"][event] then
                valuebar.Active = true
            end
        end))
    end

    if tablelength(data["End_Keyword"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["End_Keyword"][event] then
                valuebar.Active = false
            end
        end))
    end

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if valuebar.Active then
                local value = data["EventTrigger"][event]
                if value then
                    GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, self.args[event])
                end
                
            end
        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if valuebar.Active then
                local name = effect:GetName()
                local value = data["EffectAdded"][name]
                if value then
                    local args = value.Args and {
                        ["Duration"] = effect:GetDuration(),
                        ["Category"] = effect:GetCategory(),
                        ["Icon"] = effect:GetIcon(),
                        ["IsCurable"] = effect:IsCurable(),
                        ["IsDebuff"] = effect:IsDebuff(),
                        ["IsDispellable"] = effect.IsDispellable and effect:IsDispellable(),
                        ["StartTime"] = effect:GetStartTime(),
                        ["Description"] = effect:GetDescription(),
                        ["Effectname"] = effect:GetName(),
                    }
                    GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback(
        function(args)
            if valuebar.Active then
                local name = args.Name
                local value = data["EffectAdded_Target"][name]
                if value then
                    local args = value.Args and {
                        ["Duration"] = args.Duration,
                        ["Category"] = args.Category,
                        ["Icon"] = args.Icon,
                        ["IsCurable"] = args.IsCurable,
                        ["IsDebuff"] = args.IsDebuff,
                        ["StartTime"] = args.StartTime,
                        ["Description"] = args.Description,
                        ["Effectname"] = args.Name,
                        ["Time"] = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                    }
                    GroupVitalsHandlerObject:FireEvent(GroupVitalsHandlerObject:GetTargetName(), value, args)
                end
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            if valuebar.Active then
                local name = args.effect.Name
                local value = data["EffectAdded_Group"][name]
                if value then
                    local args = value.Args and {
                        ["Duration"] = args.effect.Duration,
                        ["Category"] = args.effect.Category,
                        ["Icon"] = args.effect.Icon,
                        ["IsCurable"] = args.effect.IsCurable,
                        ["IsDebuff"] = args.effect.IsDebuff,
                        --["IsDispellable"] = args.effect.IsDispellable and args.effect:IsDispellable(),
                        ["StartTime"] = args.effect.StartTime,
                        ["Description"] = args.effect.Description,
                        ["Name"] = args.name,
                        ["Effectname"] = args.effect.Name,
                    }
                    GroupVitalsHandlerObject:FireEvent(args.name, value, args)
                end
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if valuebar.Active then
                local name = effect:GetName()
                local value = data["EffectRemoved"][name]
                if value then
                    local args = value.Args and {
                        ["Duration"] = effect:GetDuration(),
                        ["Category"] = effect:GetCategory(),
                        ["Icon"] = effect:GetIcon(),
                        ["IsCurable"] = effect:IsCurable(),
                        ["IsDebuff"] = effect:IsDebuff(),
                        ["IsDispellable"] = effect.IsDispellable and effect:IsDispellable(),
                        ["StartTime"] = effect:GetStartTime(),
                        ["Description"] = effect:GetDescription(),
                        ["Effectname"] = effect:GetName(),
                    }
                    GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                end
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        valuebar.StandardPatternMatching = {}
        for key, value in pairs(data["StandardChat"]) do
            if value.Pattern_Matching then
                valuebar.StandardPatternMatching[key] = value
            end
        end

        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            if valuebar.Active then
                local value = data["StandardChat"][message]
                if value then
                    GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                end
                
                self:PatternMatchFunctionGroup(valuebar.StandardPatternMatching, args)
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        valuebar.PartyPatternMatching = {}
        for key, value in pairs(data["PartyChat"]) do
            if value.Pattern_Matching then
                valuebar.PartyPatternMatching[key] = value
            end
        end

        table.insert(self.partyInfoChatCallback, AddCallback(ChatEventHandler, "PartyInfo", function(args)
            if valuebar.Active then
                local value = data["PartyChat"][args.Message]
                if value then
                    GroupVitalsHandlerObject:FireEvent(args.Source, value, args)
                end

                self:PatternMatchFunctionGroup(valuebar.PartyPatternMatching, args)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        valuebar.SayPatternMatching = {}
        for key, value in pairs(data["SayChat"]) do
            if value.Pattern_Matching then
                valuebar.SayPatternMatching[key] = value
            end
        end

        table.insert(self.sayInfoChatCallback, AddCallback(ChatEventHandler, "SayInfo", function(args)
            if valuebar.Active then
                local value = data["SayChat"][args.Message]
                if value then
                    GroupVitalsHandlerObject:FireEvent(args.Source, value, args)
                end

                self:PatternMatchFunctionGroup(valuebar.SayPatternMatching, args)
            end
        end))
    end

    if tablelength(data["FightStart"]) > 0 then
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightStart"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                        end
                    end
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightPhase"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) and (value.Phase == args.Phase or value.Phase == nil) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                        end
                    end
                end
            end
        end))
    end

    if tablelength(data["FightEnd"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            if valuebar.Active then
                for key, value in pairs(data["FightEnd"]) do
                    if (value.Boss_Name == args.Boss_Name or value.Boss_Name == "All") and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) then
                        for keyword in string.gmatch(value["Event"], "([^,]+)") do
                            GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                        end
                    end
                end
            end
        end))
    end
    
    if tablelength(data["DefeatEvent"]) > 0 then
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            if valuebar.Active then
                if data["DefeatEvent"][args.Name] then
                    GroupVitalsHandlerObject:FireEvent(self.player:GetName(), value, args)
                end
            end
        end))
    end

    table.insert(self.UI, valuebar)
    -- Morale threshold
    -- Party Morale threshold
end