function PrimeUI:ResourceBar2(data, parent)

	local layoutTable = {
        Scale = data.Scale or 1,
        BorderSize = data.BorderSize or 0,
        Normal_Opacity  = data.Opacity,
        Always_Visible = data.Always_Visible,
        Icon = data.Icon,
        Box_Offset = data.Box_Offset,
        Box_BorderSize = data.Box_BorderSize,
        Text_Height = data.Text_Height,
        Text_Offset_X = data.Text_Offset_X,
        Text_Offset_Y = data.Text_Offset_Y,
    
        Text_Font = Turbine.UI.Lotro.Font[data.Font],
        Text_Font_Colour = self:ColourTupleToTurbineColour(data.Font_Colour),

		Base_Box_BackgroundColour = self:ColourTupleToTurbineColour(data.Base_Box_BackgroundColour),
        Base_Box_Colour = self:ColourTupleToTurbineColour(data.Base_Box_Colour),
        Bar_Background_Colour = self:ColourTupleToTurbineColour(data.Bar_Background_Colour),
        Bar_Colour = self:ColourTupleToTurbineColour(data.Bar_Colour),
        Border_Colour = self:ColourTupleToTurbineColour(data.Border_Colour),

        Additional_Colour_Rules_Foreground = data.Additional_Colour_Rules_Foreground,
        Additional_Colour_Rules_Background = data.Additional_Colour_Rules_Background,
    
        Timer_Alignment = Turbine.UI.ContentAlignment[data.Timer_TextAlignment],
        Text_Alignment = Turbine.UI.ContentAlignment[data.Text_TextAlignment],

        Invisible_While_Timer_Active = data.Invisible_While_Timer_Active,
        Invisible_While_Timer_Inactive = data.Invisible_While_Timer_Inactive,

        Show_Duration = data.Show_Duration,
		Show_Text = data.Show_Text,
        Text = data.Text,
		Text_X_Offset = data.Text_X_Offset,
		Text_Y_Offset = data.Text_Y_Offset,

		Duration_X_Offset = data.Duration_X_Offset,
		Duration_Y_Offset = data.Duration_Y_Offset,

		Alignment = data["Alignment"],
        Width = data.Width,
        Height = data.Height,
        X = data.X,
        Y = data.Y,

        Vertical = data.Vertical,
        Ticks = data.Ticks,
        Bar_Direction = data.Bar_Direction,

        Do_Not_Update = data.Do_Not_Update,
        VisibitilityThreshold = data.Visibitility_Threshold,
    }


	local valuebar = ResourceBar(parent, layoutTable)

    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or "valuebar")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]) + (data.OffsetX or 0), self.currentHeight)
    end

    self.lookup[data.NickName or ""] = valuebar

    if self.playerClass == Turbine.Gameplay.Class.Champion then
        valuebar:SetValue(self.ClassAttr:GetFervor())
        if not(self.attrCallbacks["FervorChanged"] ) then self.attrCallbacks["FervorChanged"] = {} end
        table.insert(self.attrCallbacks["FervorChanged"], AddCallback(self.ClassAttr, "FervorChanged", function(sender, args) valuebar:SetValue( self.ClassAttr:GetFervor() ) end))
    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
        valuebar:SetValue(self.ClassAttr:GetFocus())
        if not(self.attrCallbacks["FocusChanged"] ) then self.attrCallbacks["FocusChanged"] = {} end
        table.insert(self.attrCallbacks["FocusChanged"], AddCallback(self.ClassAttr, "FocusChanged", function(sender, args) valuebar:SetValue( self.ClassAttr:GetFocus() ) end))
    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
        if not(self.attrCallbacks["AttunementChanged"] ) then self.attrCallbacks["AttunementChanged"] = {} end
        valuebar:SetAttunement(self.ClassAttr:GetAttunement())
        table.insert(self.attrCallbacks["AttunementChanged"], AddCallback(self.ClassAttr, "AttunementChanged", function(sender, args) valuebar:SetAttunement(self.ClassAttr:GetAttunement()) end))
    elseif self.playerClass == (Turbine.Gameplay.Class.Mariner or Turbine.Gameplay.Class.Corsair or Turbine.Gameplay.Class.Chicken) then
        valuebar:SetBalance(self.ClassAttr:GetBalance())
        if not(self.attrCallbacks["BalanceChanged"] ) then self.attrCallbacks["BalanceChanged"] = {} end
        table.insert(self.attrCallbacks["BalanceChanged"], AddCallback(self.ClassAttr, "BalanceChanged", function(sender, args) valuebar:SetBalance(self.ClassAttr:GetBalance()) end))
    elseif self.playerClass == Turbine.Gameplay.Class.Brawler then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                valuebar:SetValue( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.gsub(effect:GetName(), "%d", "") == "Battle Flow " then
                valuebar:SetValue( tonumber(string.gsub(effect:GetName(), "%D", "")) )
                self.battleFlowID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.battleFlowID == effect:GetID() then
                valuebar:SetValue(0)
            end
        end))

    elseif self.playerClass == Turbine.Gameplay.Class.Warden then
        valuebar.resourceTable = {
            ["Advanced Technique I"] = 1,
            ["Advanced Technique II"] = 2,
            ["Advanced Technique III"] = 3,
            ["Advanced Technique IV"] = 4,
            ["Advanced Technique V"] = 5,
        }
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if valuebar.resourceTable[effect:GetName()] then
                valuebar:SetValue(valuebar.resourceTable[effect:GetName()])
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if valuebar.resourceTable[effect:GetName()] then
                valuebar:SetValue( valuebar.resourceTable[effect:GetName()] )
                valuebar.ATID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if valuebar.ATID == effect:GetID() then
                valuebar:SetValue(0)
            end
        end))

    elseif self.playerClass == Turbine.Gameplay.Class.Guardian then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if self.fortificationTable[effect:GetName()] then
                valuebar:SetValue(self.fortificationTable[effect:GetName()])
                self.forticationID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.forticationID == effect:GetID() and self.fortificationTable[effect:GetName()] then
                valuebar:SetValue(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.Beorning then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.match(effect:GetName(), "Guarded %- Tier (.*)") then
                self.GuardedTierID = effect:GetID()
                valuebar:SetValue(tonumber(string.match(effect:GetName(), "Guarded %- Tier (.*)")))
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.match(effect:GetName(), "Guarded %- Tier (.*)") then
                self.GuardedTierID = effect:GetID()
                valuebar:SetValue(tonumber(string.match(effect:GetName(), "Guarded %- Tier (.*)")))
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.GuardedTierID == effect:GetID() then
                valuebar:SetValue(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.BlackArrow then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.find(effect:GetName(), "Keen Eye") then
                self.keenEyeID = effect:GetID()
                if string.match(effect:GetName(), "Keen Eye %((.*)%)") then
                    valuebar:SetValue(tonumber(string.match(effect:GetName(), "Keen Eye %((.*)%)")))
                elseif effect:GetName() == "Keen Eye" then
                    valuebar:SetValue(1)
                end
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.find(effect:GetName(), "Keen Eye") then
                self.keenEyeID = effect:GetID()
                if string.match(effect:GetName(), "Keen Eye %((.*)%)") then
                    valuebar:SetValue(tonumber(string.match(effect:GetName(), "Keen Eye %((.*)%)")))
                elseif effect:GetName() == "Keen Eye" then
                    valuebar:SetValue(1)
                end
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.keenEyeID == effect:GetID() then
                valuebar:SetValue(0)
            end
        end))
    elseif self.playerClass == Turbine.Gameplay.Class.Weaver then
        for i = 1, self.player:GetEffects():GetCount(), 1 do
            local effect = self.player:GetEffects():Get(i)
            if string.gsub(effect:GetName(), "%d", "") == "Venom " then
                valuebar:SetValue( tonumber(string.gsub(effect:GetName(), "%D", "")) )
            end
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            if string.gsub(effect:GetName(), "%d", "") == "Venom " then
                valuebar:SetValue( tonumber(string.gsub(effect:GetName(), "%D", "")) )
                self.venomID = effect:GetID()
            end
        end))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            if self.venomID == effect:GetID() then
                valuebar:SetValue(0)
            end
        end))
    end

    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
    self.heightSize = self.heightSize + data["Height"] + 2 * data["BorderSize"]
    table.insert(self.tools, valuebar)
end