function PrimeUI:Timer(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"], data["Height"], data["NickName"] or "Timer")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"]), self.currentHeight)
    end
    valuebar:SetSize(data["Width"], data["Height"])
    valuebar:SetMouseVisible(false)

    local currentHeight = 0
    local colourScheme = {self:ColourTupleToTurbineColour(data["BackgroundColour"]), self:ColourTupleToTurbineColour(data["BackgroundColour"]), self:ColourTupleToTurbineColour(data["ForegroundColour"]), Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), nil, nil, nil, 1, 1}

    local timer = EffectTimer(valuebar, data["Width"], data["Height"], nil, data["Text"], self:FontToFont(data["Font"]), colourScheme, true, nil, 0, data["Icon"], nil, not(data["AlwaysVisible"]), self:FontToFont(data["Duration_Font"]), data["Spark"], data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]))
    timer:SetPosition(data["BorderSize"], data["BorderSize"])
    if not(data["AlwaysVisible"]) then
        timer:SetVisible(false)
    end

    self.lookup[data.NickName or ""] = timer
    timer.highlight = ClassSpecificTools.HighlightFrame2(timer, data["Height"] / 32)
    timer.highlight:Toggle(false) 

    timer.originalMessage = data["Text"]
    timer.currentEffect = ""
    timer.SkillID = {}
    timer:SetOpacity(data.Opacity_While_Inactive or 1)
    timer.effectIcon:SetOpacity(data.Opacity_While_Inactive or 1)
    timer.textLabel:SetVisible(not data.Hide_Text_While_Inactive)
    --timer:ChangeBackOpacity(0.4)
    --timer:ChangeFrontOpacity(0.4)
    
    timer.nameFunction = function(text, name, args)
        return ReplaceTokens(string.gsub(text, "<name>", NameHandlerObject:GetName(name, true, "PrimeUITimer", "")), args or {})
    end

    timer.timerFunction = function(value, time, icon, args)
        local highlight = data["AllHighlight"] or value["Highlight"]
        timer.highlight:Toggle(highlight) 
        timer:SetTimer(time or value["Time"] or 0, time or value["Time"] or 0)
        self.timers[timer] = {["Hide_Text_While_Inactive"] = data.Hide_Text_While_Inactive, ["Opacity_While_Inactive"] = data.Opacity_While_Inactive or 1}
        timer:ChangeText(timer.nameFunction(timer.originalMessage, args and args.name or self.player:GetName(), args))
        if not(self.updateCallback) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
        if time and time > 0 or value["Time"] then
            timer:SetOpacity(data.Opacity_While_Active  or 1)
            timer.effectIcon:SetOpacity(data.Opacity_While_Active or 1)
            timer.textLabel:SetVisible(true)
        else
            timer:SetOpacity(data.Opacity_While_Inactive or 1)
            timer.effectIcon:SetOpacity(data.Opacity_While_Inactive or 1)
            timer.textLabel:SetVisible(not data.Hide_Text_While_Inactive)
        end

        if value.Change_Text then
            if args and not args.name then args.name = self.player:GetName() end
            timer:ChangeText(timer.nameFunction(value.Change_Text_Text or "", args and args.name or self.player:GetName(), args))
        end

        if value.Change_Icon then
            timer:ChangeIcon(value.Change_Icon_Icon)
        end

        if value.Use_Custom_Colour then
            timer.timerBar:SetBackColor(Turbine.UI.Color((tonumber(value.Custom_R) or 1)/255, (tonumber(value.Custom_G) or 1)/255, (tonumber(value.Custom_B) or 1)/255 ))
        else
            timer.timerBar:SetBackColor(self:ColourTupleToTurbineColour(data["ForegroundColour"]))
        end
    end

    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == event then
                        return
                    else
                        timer.currentEffect = event
                    end
                end
                
                timer.timerFunction(value, value.Time == true and self.args[event] and (tonumber(self.args[event]["Match$1"]) or tonumber(self.args[event]["Match$2"]) or tonumber(self.args[event]["Match$3"]) or tonumber(self.args[event]["Match$4"]) or tonumber(self.args[event]["Match$5"])), nil, self.args[event])
            end
        end))
    end

    if tablelength(data["Effects"]) > 0 then
        timer.effectFunction = function(effect)
            local name = effect:GetName()
            if data["Effects"][name] then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == effect:GetName() then
                        return
                    else
                        timer.currentEffect = effect:GetName()
                    end
                end
                local value = data["Effects"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration()
                end

                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                end

                timer.SkillID[effect:GetID()] = true
                timer.timerFunction(value, time, icon)
            end
        end

        for i = 1, self.player:GetEffects():GetCount() do
            timer.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", timer.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if timer.SkillID[ID] then
                timer:SetTimer(0, 0)
            end

        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] ~= nil then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == effect:GetName() then
                        return
                    else
                        timer.currentEffect = effect:GetName()
                    end
                end
                local value = data["EffectAdded"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration()
                end

                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                end

                timer.timerFunction(value, time, icon)
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local effectArgs = args.effect
            local name = effectArgs.Name
            if data["EffectAdded_Group"][name] ~= nil then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == effectArgs.Name then
                        return
                    else
                        timer.currentEffect = effectArgs.Name
                    end
                end
                local value = data["EffectAdded_Group"][name]
                local time = value["Time"]
                if value["Time"] == true then
                    time = effectArgs.Duration
                end

                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = effectArgs.Icon
                end

                timer.timerFunction(value, time, icon, args)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] ~= nil then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.Name then
                        return
                    else
                        timer.currentEffect = args.Name
                    end
                end
                local value = data["EffectAdded_Target"][name]
                local time = value["Time"]
                if time == true then
                    time = args.Duration
                end

                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = args.Icon
                end

                timer.timerFunction(value, time, icon, args)
            end
        end))
    end

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            if data["EffectRemoved"][name] ~= nil then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == effect:GetName() then
                        return
                    else
                        timer.currentEffect = effect:GetName()
                    end
                end
                local value = data["EffectRemoved"][name]
                local time = value["Time"]
                if time == true then
                    time = effect:GetDuration()
                end

                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                end

                timer.timerFunction(value, time, icon, args)
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill)   
                    if data["Unique"] and timer.currentEffect then
                        if timer.currentEffect == skill:GetSkillInfo():GetName() then
                            return
                        else
                            timer.currentEffect = skill:GetSkillInfo():GetName()
                        end
                    end
                    local time = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                    end

                    if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= skill:GetCooldown() then
                        return
                    end

                    local icon = value["Icon"]
                    if value["Icon"] == true then
                        icon = skill:GetSkillInfo():GetIconImageID()
                    end

                    timer.timerFunction(value, time, icon)
                end))
            end
        end
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                if args.Failed and not value.Also_Trigger_On_Resist and not value.Only_Trigger_On_Resist then return end
                if not args.Failed and value.Only_Trigger_On_Resist then return end
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.SkillName then
                        return
                    else
                        timer.currentEffect = args.SkillName
                    end
                end
                timer.timerFunction(value, nil, nil, args)
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                if args.Failed and not value.Also_Trigger_On_Resist and not value.Only_Trigger_On_Resist then return end
                if not args.Failed and value.Only_Trigger_On_Resist then return end
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.SkillName then
                        return
                    else
                        timer.currentEffect = args.SkillName
                    end
                end
                
                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.SkillName then
                        return
                    else
                        timer.currentEffect = args.SkillName
                    end
                end
                
                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.SkillName then
                        return
                    else
                        timer.currentEffect = args.SkillName
                    end
                end
                
                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            local value = data["Action"][args.Action]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.Action then
                        return
                    else
                        timer.currentEffect = args.Action
                    end
                end
                
                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == message then
                        return
                    else
                        timer.currentEffect = message
                    end
                end

                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "Party", function(message)
            local value = data["PartyChat"][message]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == message then
                        return
                    else
                        timer.currentEffect = message
                    end
                end

                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["DefeatEvent"]) > 0 then
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            local value = data["DefeatEvent"][args.Name] 
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.Name then
                        return
                    else
                        timer.currentEffect = args.Name
                    end
                end

                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayInfo", function(args)
            local value = data["SayChat"][args.Message]
            if value then
                if data["Unique"] and timer.currentEffect then
                    if timer.currentEffect == args.Message then
                        return
                    else
                        timer.currentEffect = args.Message
                    end
                end

                timer.timerFunction(value, nil, nil, args)
            end
        end))
    end

    table.insert(self.tools, timer)

    
    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
    self.heightSize = self.heightSize + data["Height"] + 2 * data["BorderSize"]
    table.insert(self.UI, valuebar)
end