function PrimeUI:TimerGroup(data, parent)
    local valuebar = Turbine.UI.Control()
    valuebar:SetParent(parent)
    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"], data["Height"], data["NickName"] or "Timer Group")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"]), self.currentHeight)
    end
    valuebar:SetSize(data["Width"], data["Height"])
    valuebar:SetMouseVisible(false)

    valuebar.nameFunction = function(text, name, args)
        return ReplaceTokens(string.gsub(text or "", "<name>", NameHandlerObject:GetName(name, true, "PrimeUITimerGroup", "") or ""), args or {})
    end

    local innerbar = Turbine.UI.Control()
    innerbar:SetParent(valuebar)
    innerbar:SetPosition(data["BorderSize"], data["BorderSize"])
    innerbar:SetSize(data["Width"], data["Height"])
    innerbar:SetMouseVisible(false)
    innerbar:SetBackColor(self:ColourTupleToTurbineColour(data["BackgroundColour"]))

    local currentHeight = 0
    local colourScheme = {self:ColourTupleToTurbineColour(data["Timer_BackgroundColour"]), self:ColourTupleToTurbineColour(data["Timer_BackgroundColour"]), self:ColourTupleToTurbineColour(data["Timer_ForegroundColour"]), Turbine.UI.Color(1,1,1), Turbine.UI.Color(0,0,0), self:ColourTupleToTurbineColour(data.Warning_Colour), nil, nil, 1, 1}

    local timerGroup = TimerGroupWindow(innerbar, data["Width"], data["Height"], 0, 0, data["Timer_Sorting"], data["Timer_Height"], self:FontToFont(data["Timer_Font"]), self:FontToFont(data["Duration_Font"]), colourScheme, data["Static_Timers"], data["Timer_Offset"], data["Timer_Stack_Direction"], data["Spark"], data["BorderSize"], self:ColourTupleToTurbineColour(data["BorderColour"]), data["Opacity"], data.Timer_Header, data.Header_Text, data.Header_Only_When_Active, self:ColourTupleToTurbineColour(data.Header_Background_Colour), data.Name_Filter_List, data["Use Name_Filter_List"], data.Use_Warning_Colour, data.Warning_Threshold)
    timerGroup.targetMaxHealth = 0
    timerGroup.TargetName = ""
    timerGroup.SkillID = {}
    timerGroup.text_inactive_opacity  = data.Hide_Text_While_Inactive or true
    timerGroup.inactive_opacity = data.Opacity_While_Inactive or 1
    timerGroup.active_opacity = data.Opacity_While_Active or 1

    self.lookup[data.NickName or ""] = timerGroup
    if data["Reset_On_Fight_End"] then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            timerGroup:Clear()
        end))
    end

    if data["Static_Timers"] then
        if data["EffectAdded"] then
            for key, value in pairs(data["EffectAdded"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["Effects"] then
            for key, value in pairs(data["Effects"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["EffectAdded_Target"] then
            for key, value in pairs(data["EffectAdded_Target"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["EffectAdded_Group"] then
            for key, value in pairs(data["EffectAdded_Group"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["EffectRemoved"] then
            for key, value in pairs(data["EffectRemoved"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["Skills"] then
            for key, value in pairs(data["Skills"]) do
                local icon = value["Icon"]
                if value["Icon"] == true then
                    icon = 0x41007F41
                end
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], icon, self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["EnemyHits"] then
            for key, value in pairs(data["EnemyHits"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["PlayerHits"] then
            for key, value in pairs(data["PlayerHits"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end
        
        if data["PlayerBenefit"] then
            for key, value in pairs(data["PlayerBenefit"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end
        
        if data["PlayerHeal"] then
            for key, value in pairs(data["PlayerHeal"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["StandardChat"] then
            for key, value in pairs(data["StandardChat"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["PartyChat"] then
            for key, value in pairs(data["PartyChat"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["SayChat"] then
            for key, value in pairs(data["SayChat"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["FightStart"] then
            for key, value in pairs(data["FightStart"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["FightPhase"] then
            for key, value in pairs(data["SayChat"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["FightEnd"] then
            for key, value in pairs(data["SayChat"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["Target_HealthThreshold"] then
            for key, value in pairs(data["Target_HealthThreshold"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["EventTrigger"] then
            for key, value in pairs(data["EventTrigger"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["DefeatEvent"] then
            for key, value in pairs(data["DefeatEvent"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["DNFApplied"] then
            for key, value in pairs(data["DNFApplied"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["DNFRemoved"] then
            for key, value in pairs(data["DNFRemoved"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["Action"] then
            for key, value in pairs(data["Action"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end

        if data["CCBreak"] then
            for key, value in pairs(data["CCBreak"]) do
                timerGroup:NewTimer(0, valuebar.nameFunction(value["Message"], self.player:GetName()), value["Message"], value["Unique"], value["Icon"], self.player:GetName(), true, nil, value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
            end
        end
    end

    if tablelength(data["Start_Keyword"]) > 0 then
        timerGroup:SetupStartmode(true)
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Start_Keyword"][event] then
                timerGroup:ToggleStartValue(true)
            end
        end))
    end

    if tablelength(data["End_Keyword"]) > 0 then
        timerGroup:SetupStartmode(true)
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["End_Keyword"][event] then
                timerGroup:ToggleStartValue(false)
            end
        end))
    end

    if tablelength(data["Reset_Keyword"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            if data["Reset_Keyword"][event] then
                timerGroup:Clear()
            end
        end))
    end
    
    if tablelength(data["EventTrigger"]) > 0 then
        table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
        function(event)
            local value = data["EventTrigger"][event]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = self.player:GetName(),
                    Source = self.player:GetName(),
                    -- Naming replacement args [external]
                    target = self.player:GetName(),
                    name = self.player:GetName(),
                }
                if self.args[event] and type(self.args[event]) == "table" then
                    for k, v in pairs(self.args[event]) do
                        args[k] = v
                    end
                end
                timerGroup.timerFunction(args)
            end
        end))
    end

    timerGroup.staticTimerFunction = function(args)
        return timerGroup:StartStaticTimer(
            args.Time, 
            args.OriginalMessage, 
            valuebar.nameFunction(args.Message, args.Trigger_Specific_Name or self.player:GetName(), args), 
            args.Source,
            args.Icon, 
            args.IconValue, 
            args.Unique, 
            args.Max_Time or args.Time,
            {
                Custom_Colour = args.Use_Custom_Colour,
                Custom_R = args.Custom_R,
                Custom_G = args.Custom_G,
                Custom_B = args.Custom_B,
            })
    end

    timerGroup.normalTimerFunction = function(args)
        return timerGroup:NewTimer(
                args.Time,
                valuebar.nameFunction(args.Message, args.Trigger_Specific_Name or self.player:GetName(), args),
                args.Message,
                args.Unique,
                args.Icon,
                args.Source,
                false,
                args.Override,
                args.Callback_Function,
                nil,
                args.Increasing,
                {
                    Custom_Colour = args.Use_Custom_Colour,
                    Custom_R = args.Custom_R,
                    Custom_G = args.Custom_G,
                    Custom_B = args.Custom_B,
                })
    end

    timerGroup.timerFunction = function(args)
        if args.Event_Keyword then
            if not(self.args[args.Event_Keyword]) then
                self.args[args.Event_Keyword] = {}
            end
            self.args[args.Event_Keyword] = args
        end

        if data.Static_Timers then
            return timerGroup.staticTimerFunction(args)
        else
            return timerGroup.normalTimerFunction(args)
        end
    end

    if tablelength(data["Effects"]) > 0 then
        timerGroup.effectFunction = function(effect)
            local name = effect:GetName()
            local value = data["Effects"][name]
            if value then
                local time = value["Time"]
                local maxTime = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    maxTime = effect:GetDuration()
                end
        
                local icon = value["Icon"]
                local iconValue = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                    iconValue = 0x41007F41
                elseif value["Match_icon"] then
                    if tostring(icon) ~= tostring(effect:GetIcon()) then
                        return
                    end
                end

                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,

                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,


                    -- Timer value args
                    Time = time,
                    Max_Time = maxTime,
                    Icon = icon,
                    IconValue = iconValue,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = self.player:GetName(),
                    Source = self.player:GetName(),
                    -- Naming replacement args [external]
                    effectname = effect:GetName(),
                    target = self.player:GetName(),
                    name = self.player:GetName(),
                }
                timerGroup.SkillID[effect:GetID()] = timerGroup.timerFunction(args)
            end
        end

        for i = 1, self.player:GetEffects():GetCount() do
            timerGroup.effectFunction(self.player:GetEffects():Get(i))
        end

        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", timerGroup.effectFunction))

        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local ID = effect:GetID()
            if timerGroup.SkillID[ID] then
                timerGroup:DeleteTimer(timerGroup.SkillID[ID])
            end

        end))
    end

    if tablelength(data["EffectAdded"]) > 0 then
        table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
        function(effect)
            local name = effect:GetName()
            if data["EffectAdded"][name] then
                local value = data["EffectAdded"][name]
                local time = value["Time"]
                local maxTime = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    maxTime = effect:GetDuration()
                end
        
                local icon = value["Icon"]
                local iconValue = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                    iconValue = 0x41007F41
                elseif value["Match_icon"] then
                    if tostring(icon) ~= tostring(effect:GetIcon()) then 
                        return 
                    end
                end

                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = time,
                    Max_Time = maxTime,
                    Icon = icon,
                    IconValue = iconValue,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = self.player:GetName(),
                    Source = self.player:GetName(),
                    -- Naming replacement args [external]
                    effectname = effect:GetName(),
                    target = self.player:GetName(),
                    name = self.player:GetName(),
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["EffectAdded_Target"]) > 0 then
        timerGroup.skillsTargeted = true
        table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
        function(args)
            local name = args.Name
            if data["EffectAdded_Target"][name] ~= nil then
                local value = data["EffectAdded_Target"][name]
                local time = value["Time"]
                local maxTime = value["Time"]
                if value["Time"] == true then
                    time = args.Duration - (Turbine.Engine.GetGameTime() - args.StartTime)
                    maxTime = args.Duration
                else
                    maxTime = time
                    time = time - (Turbine.Engine.GetGameTime() - args.StartTime)
                end

                local icon = value["Icon"]
                local iconValue = value["Icon"]
                if value["Icon"] == true then
                    icon = args.Icon
                    iconValue = 0x41007F41
                elseif value["Match_icon"] then
                    if tostring(icon) ~= tostring(args.Icon) then return end
                end

                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = time,
                    Max_Time = maxTime,
                    Icon = icon,
                    IconValue = iconValue,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Target,
                    Source = args.Target,
                    -- Naming replacement args [external]
                    effectname = name,
                    target = args.Target,
                    name = args.Target,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["EffectAdded_Group"]) > 0 then
        table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
        function(args)
            local effectArgs = args.effect
            local name = effectArgs.Name
            local value = data["EffectAdded_Group"][name]
            if value then
                local time = value["Time"]
                local maxTime = value["Time"]
                if value["Time"] == true then
                    time = effectArgs.Duration - (Turbine.Engine.GetGameTime() - effectArgs.StartTime)
                    maxTime = effectArgs.Duration
                end

                local icon = value["Icon"]
                local iconValue = value["Icon"]
                if value["Icon"] == true then
                    icon = effectArgs.Icon
                    iconValue = 0x41007F41
                elseif value["Match_icon"] then
                    if tostring(icon) ~= tostring(effectArgs.Icon) then return end
                end

                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = time,
                    Max_Time = maxTime,
                    Icon = icon,
                    IconValue = iconValue,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.name,
                    Source = args.name,
                    -- Naming replacement args [external]
                    effectname = name,
                    target = args.name,
                    name = args.name,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    

    if tablelength(data["EffectRemoved"]) > 0 then
        table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
        function(effect)
            local name = effect:GetName()
            local value = data["EffectRemoved"][name]
            if value then
                local time = value["Time"]
                local maxTime = value["Time"]
                if value["Time"] == true then
                    time = effect:GetDuration() - (Turbine.Engine.GetGameTime() - effect:GetStartTime())
                    maxTime = effect:GetDuration()
                end

                local icon = value["Icon"]
                local iconValue = value["Icon"]
                if value["Icon"] == true then
                    icon = effect:GetIcon()
                    iconValue = 0x41007F41
                elseif value["Match_icon"] then
                    if tostring(icon) ~= tostring(effect:GetIcon()) then return end
                end

                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = time,
                    Max_Time = maxTime,
                    Icon = icon,
                    IconValue = iconValue,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = self.player:GetName(),
                    Source = self.player:GetName(),
                    -- Naming replacement args [external]
                    effectname = effect:GetName(),
                    target = self.player:GetName(),
                    name = self.player:GetName(),
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["Skills"]) > 0 then
        self.skills = true
        for key, value in pairs(data["Skills"]) do
            if SkillEventHandler:SkillExists(key) then
                if string.find(value["Message"], "<name>") then
                    --timerGroup.skillsTargeted = true
                end
                -- Add the callback for this skill slot
                if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill)  
                    local time = value["Time"]
                    local maxTime = value["Time"]
                    if value["Time"] == true then
                        time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                        maxTime = skill:GetCooldown()
                    end

                    if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= maxTime then
                        return
                    end

                    local icon = value["Icon"]
                    local iconValue = value["Icon"]
                    if value["Icon"] == true then
                        icon = skill:GetSkillInfo():GetIconImageID()
                        iconValue = 0x41007F41
                    end

                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
    
                        -- Timer value args
                        Time = time,
                        Max_Time = maxTime,
                        Icon = icon,
                        IconValue = iconValue,
                        Message = value.Message,
                    OriginalMessage = value.Message,
    
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = self.player:GetName(),
                        Source = self.player:GetName(),
                        -- Naming replacement args [external]
                        skillname = key,
                        name = self.player:GetName(),
                    }
                    timerGroup.timerFunction(args)
                end))
            end
        end
    end

    if tablelength(data["Action"]) > 0 then
        table.insert(self.actionCallbacks, AddCallback(ActionHandlerObject, "Action", function(args)
            local value = data["Action"][args.Action]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = self.player:GetName(),
                    Source = self.player:GetName(),
                    -- Naming replacement args [external]
                    target = self.player:GetName(),
                    name = self.player:GetName(),
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["EnemyHits"]) > 0 then
        table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyCombat", function(args)
            local value = data["EnemyHits"][args.SkillName]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Target,
                    Source = args.Source or args.Target,
                    -- Naming replacement args [external]
                    skillname = args.SkillName,
                    source = args.Source,
                    target = args.Target,
                    name = args.Target,
                }
                timerGroup.timerFunction(args)
            end
        end))

    end

    if tablelength(data["PlayerHits"]) > 0 then
        table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerCombat", function(args)
            local value = data["PlayerHits"][args.SkillName]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Target,
                    Source = args.Source or args.Target,
                    -- Naming replacement args [external]
                    skillname = args.SkillName,
                    source = args.Source,
                    target = args.Target,
                    name = args.Target,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["PlayerHeal"]) > 0 then
        table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", function(args)
            local value = data["PlayerHeal"][args.SkillName]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Target,
                    Source = args.Source or args.Target,
                    -- Naming replacement args [external]
                    skillname = args.SkillName,
                    source = args.Source,
                    target = args.Target,
                    name = args.Target,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["PlayerBenefit"]) > 0 then
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", function(args)
            local value = data["PlayerBenefit"][args.SkillName]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Target,
                    Source = args.Source or args.Target,
                    -- Naming replacement args [external]
                    skillname = args.SkillName,
                    source = args.Source,
                    target = args.Target,
                    name = args.Target,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["StandardChat"]) > 0 then
        table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", function(message)
            local value = data["StandardChat"][message]
            if value then
                if value then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
    
                        -- Timer value args
                        Time = value.Time,
                        Max_Time = value.Time,
                        Icon = value.Icon,
                        IconValue = value.Icon,
                        Message = value.Message,
                    OriginalMessage = value.Message,
    
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = self.player:GetName(),
                        Source = self.player:GetName(),
                        -- Naming replacement args [external]
                        source = self.player:GetName(),
                        name = self.player:GetName(),
                    }
                    timerGroup.timerFunction(args)
                end
            end
        end))
    end

    if tablelength(data["PartyChat"]) > 0 then
        table.insert(self.partyChatCallback, AddCallback(ChatEventHandler, "PartyInfo", function(args)
            local value = data["PartyChat"][args.Message]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Source,
                    Source = args.Source,
                    -- Naming replacement args [external]
                    source = args.Source,
                    name = args.Source,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["SayChat"]) > 0 then
        table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayInfo", function(args)
            local value = data["SayChat"][args.Message]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Source,
                    Source = args.Source,
                    -- Naming replacement args [external]
                    source = args.Source,
                    name = args.Source,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["DefeatEvent"]) > 0 then
        table.insert(self.defeatEventCallback, AddCallback(ChatEventHandler, "Defeated", function(args)
            local value = data["DefeatEvent"][args.Name]
            if value then
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,

                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,

                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Name,
                    Source = args.Source,
                    -- Naming replacement args [external]
                    source = args.Source,
                    target = args.Name,
                    name = args.Name,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["FightStart"]) > 0 then
        table.insert(self.fightEventStartCallbacks, AddCallback(InstanceController, "FightEventStart", function(args)
            for key, value in pairs(data["FightStart"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or  tostring(value.Tier_Info) == args.Boss_Tier) then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
    
                        -- Timer value args
                        Time = value.Time,
                        Max_Time = value.Time,
                        Icon = value.Icon,
                        IconValue = value.Icon,
                        Message = value.Message,
                    OriginalMessage = value.Message,
    
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = args.Boss_Name,
                        Source = args.Boss_Name,
                        -- Naming replacement args [external]
                        source = args.Boss_Name,
                        tier = args.Boss_Tier,
                        name = args.Boss_Name,
                    }
                    timerGroup.timerFunction(args)
                end
            end
        end))
    end

    if tablelength(data["FightPhase"]) > 0 then
        table.insert(self.fightEventPhaseCallback, AddCallback(InstanceController, "FightEventPhase", function(args)
            for key, value in pairs(data["FightPhase"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or tostring(value.Tier_Info) == args.Boss_Tier) and value.Phase == args.Phase then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
    
                        -- Timer value args
                        Time = value.Time,
                        Max_Time = value.Time,
                        Icon = value.Icon,
                        IconValue = value.Icon,
                        Message = value.Message,
                    OriginalMessage = value.Message,
    
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = args.Boss_Name,
                        Source = args.Boss_Name,
                        -- Naming replacement args [external]
                        source = args.Boss_Name,
                        tier = args.Boss_Tier,
                        name = args.Boss_Name,
                        phase = args.Phase,
                    }
                    timerGroup.timerFunction(args)
                end
            end
        end))
    end

    if tablelength(data["FightEnd"]) > 0 then
        table.insert(self.fightEventEndCallbacks, AddCallback(InstanceController, "FightEventEnd", function(args)
            for key, value in pairs(data["FightEnd"]) do
                if value.Boss_Name == args.Boss_Name and (value.Tier_Info == true or  tostring(value.Tier_Info) == args.Boss_Tier) then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
    
                        -- Timer value args
                        Time = value.Time,
                        Max_Time = value.Time,
                        Icon = value.Icon,
                        IconValue = value.Icon,
                        Message = value.Message,
                    OriginalMessage = value.Message,
    
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = args.Boss_Name,
                        Source = args.Boss_Name,
                        -- Naming replacement args [external]
                        source = args.Boss_Name,
                        tier = args.Boss_Tier,
                        name = args.Boss_Name,
                    }
                    timerGroup.timerFunction(args)
                end
            end
        end))
    end

    if tablelength(data["DNFApplied"]) > 0 then
        timerGroup.dnfApplicationFunction = function(value, args)
            local time = value.Time
            if time and time == true then
                time = args.Time
            end
            local args = {
                -- Timer settings args
                Unique = value.Unique,
                Override = value.Override,
                Increasing = value.Increasing,
                Event_Keyword = value.Event_Keyword,
                Callback_Function = value["Event_Keyword"] and function()
                    self:FireTrigger(value["Event_Keyword"])
                end,

                -- Timer value args
                Time = time,
                Max_Time = value.Time,
                Icon = value.Icon,
                IconValue = value.Icon,
                Message = value.Message,

                -- Naming replacement args [internal]
                Trigger_Specific_Name = args.Name,
                Source = args.Name,
                -- Naming replacement args [external]
                source = args.Name,
                name = args.Name,
            }
            timerGroup.timerFunction(args)
        end

        for name, time in pairs(rezTracker:GetDNFInfo()) do
            local args = {
                ["Name"] = name,
                ["Time"] = time - Turbine.Engine.GetGameTime(),
            }
            for key, value in pairs(data["DNFApplied"]) do
                timerGroup.dnfApplicationFunction(value, args)
            end
        end

        table.insert(self.dnfAppliedCallbacks, AddCallback(rezTracker, "DNFApplication", function(args)
            for key, value in pairs(data["DNFApplied"]) do
                timerGroup.dnfApplicationFunction(value, args)
            end
        end))
    end

    if tablelength(data["DNFRemoved"]) > 0 then
        table.insert(self.dnfRemovedCallbacks, AddCallback(rezTracker, "DNFRemoval", function(args)
            for key, value in pairs(data["DNFRemoved"]) do
                local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                    -- Timer settings args
                    Unique = value.Unique,
                    Override = value.Override,
                    Increasing = value.Increasing,
                    Event_Keyword = value.Event_Keyword,
                    Callback_Function = value["Event_Keyword"] and function()
                        self:FireTrigger(value["Event_Keyword"])
                    end,
    
                    -- Timer value args
                    Time = value.Time,
                    Max_Time = value.Time,
                    Icon = value.Icon,
                    IconValue = value.Icon,
                    Message = value.Message,
                    OriginalMessage = value.Message,
    
                    -- Naming replacement args [internal]
                    Trigger_Specific_Name = args.Name,
                    Source = args.Name,
                    -- Naming replacement args [external]
                    source = args.Name,
                    name = args.Name,
                }
                timerGroup.timerFunction(args)
            end
        end))
    end

    if tablelength(data["CCBreak"]) > 0 then
        for key, value in pairs(data["CCBreak"]) do
            if key == "" then
                valuebar.CCBreakAllNames = true
            end
        end
        table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "CCBreak", function(args)
            local value = data["CCBreak"][args.Target]
            if value then
                if value.CCType == args.Type or value.CCType == true then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = value.Unique,
                        Override = value.Override,
                        Increasing = value.Increasing,
                        Event_Keyword = value.Event_Keyword,
                        Callback_Function = value["Event_Keyword"] and function()
                            self:FireTrigger(value["Event_Keyword"])
                        end,
        
                        -- Timer value args
                        Time = value.Time,
                        Max_Time = value.Time,
                        Icon = value.Icon,
                        IconValue = value.Icon,
                        Message = value.Message,
                    OriginalMessage = value.Message,
        
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = args.Target,
                        Source = args.Culprit,
                        -- Naming replacement args [external]
                        source = args.Culprit,
                        name = args.Target,
                        culprit = args.Culprit,
                        type = args.Type,
                        target = args.Target,
                    }
                    timerGroup.timerFunction(args)
                end
            end
            if valuebar.CCBreakAllNames then
                local allValue = data["CCBreak"][""]
                if allValue.CCType == args.Type or allValue.CCType == true then
                    local args = {
                    Use_Custom_Colour= value.Use_Custom_Colour,
                    Custom_R = value.Custom_R,
                    Custom_G = value.Custom_G,
                    Custom_B = value.Custom_B,
                        -- Timer settings args
                        Unique = allValue.Unique,
                        Override = allValue.Override,
                        Increasing = allValue.Increasing,
                        Event_Keyword = allValue.Event_Keyword,
                        Callback_Function = allValue["Event_Keyword"] and function()
                            self:FireTrigger(allValue["Event_Keyword"])
                        end,
        
                        -- Timer allValue args
                        Time = allValue.Time,
                        Max_Time = allValue.Time,
                        Icon = allValue.Icon,
                        IconValue = allValue.Icon,
                        Message = allValue.Message,
        
                        -- Naming replacement args [internal]
                        Trigger_Specific_Name = args.Target,
                        Source = args.Culprit,
                        -- Naming replacement args [external]
                        source = args.Culprit,
                        name = args.Target,
                        culprit = args.Culprit,
                        type = "All",
                        target = args.Target,
                    }
                    timerGroup.timerFunction(args)
                end
            end
        end))
    end

    if tablelength(data["Target_HealthThreshold"]) > 0 and data["Reset_On_Retarget"] or timerGroup.skillsTargeted then
        table.insert(self.targetInfoCallbacks, TargetHandlerObject:AddTargetInfoCallback(function(value) 
            if value then
                if value["Name"] ~= timerGroup.TargetName and data["Reset_On_Retarget"] then
                    timerGroup:Clear()
                end
                timerGroup.targetMaxHealth = value["MaxMorale"]
                timerGroup.TargetName = value["Name"]
            else
                timerGroup.targetMaxHealth = 0
                timerGroup.TargetName = ""
                if data["Reset_On_Retarget"] then 
                    timerGroup:Clear()
                end
            end
        end))
    end

    
    if tablelength(data["Target_HealthThreshold"]) > 0 then
        timerGroup.targetHealthThresholdCooldowns = {}
        timerGroup.targetHealthThresholdPercentage = {}
        local health = self.player:GetMorale() / self.player:GetMaxMorale() * 100
        for key, value in pairs(data["Target_HealthThreshold"]) do
            timerGroup.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
            timerGroup.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
        end


        table.insert(self.healthTargetCallbacks, TargetHandlerObject:AddHealthCallback(function(value)
            if timerGroup.targetMaxHealth > 0 then
                local percentage = value / timerGroup.targetMaxHealth * 100
                local time = Turbine.Engine.GetGameTime()
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    if key == timerGroup.TargetName then
                        if self.compareFunctions[value["Operator"]](percentage, value["Number"]) then
                            local id = key .. tostring(value["Number"]) .. tostring(value["Operator"])
                            if timerGroup.targetHealthThresholdCooldowns[id] < time and timerGroup.targetHealthThresholdPercentage[id] then
                                timerGroup.targetHealthThresholdPercentage[id] = false
                                if data["Static_Timers"] then
                                    timerGroup:StartStaticTimer(value["Time"], value["Message"], key, value["Icon"], value["Icon"], data["Unique"])
                                else
                                    timerGroup:NewTimer(value["Time"], valuebar.nameFunction(value["Message"], key), value["Message"], value["Unique"], value["Icon"], key, nil, value["Override"], value["Event_Keyword"] and function() self:FireTrigger(value["Event_Keyword"]) end, nil, value.Increasing) 
                                end
                                timerGroup.targetHealthThresholdCooldowns[id] = time + 6
                            end
                        else
                            timerGroup.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                        end
                    end
                end
            end
        end))
        table.insert(self.combatCallbacks, AddCallback(self.player, "InCombatChanged", function()
            if not(self.player:IsInCombat()) then
                for key, value in pairs(data["Target_HealthThreshold"]) do
                    timerGroup.targetHealthThresholdCooldowns[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = Turbine.Engine.GetGameTime() - 1000
                    timerGroup.targetHealthThresholdPercentage[key .. tostring(value["Number"]) .. tostring(value["Operator"])] = true
                end 
            end
        end))
    end

    table.insert(self.tools, timerGroup)

    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
    self.heightSize = self.heightSize + data["Height"] + 2 * data["BorderSize"]
    table.insert(self.UI, valuebar)
end