function PrimeUI:ValueBar2(data, parent)

	local layoutTable = {
        Scale = data.Scale or 1,
        BorderSize = data.BorderSize or 0,
        Normal_Opacity  = data.Opacity,
        Always_Visible = data.Always_Visible,
        Icon = data.Icon,
    
        Timer_Font = Turbine.UI.Lotro.Font[data.Timer_Font],
        Text_Font = Turbine.UI.Lotro.Font[data.Text_Font],
        Timer_Font_Colour = self:ColourTupleToTurbineColour(data.Timer_Font_Colour),
        Text_Font_Colour = self:ColourTupleToTurbineColour(data.Text_Font_Colour),

		Tick_Colour = self:ColourTupleToTurbineColour(data.Tick_Colour),
        Bar_Background_Colour = self:ColourTupleToTurbineColour(data.Bar_Background_Colour),
        Bar_Colour = self:ColourTupleToTurbineColour(data.Bar_Colour),
        Border_Colour = self:ColourTupleToTurbineColour(data.Border_Colour),
    
        Timer_Alignment = Turbine.UI.ContentAlignment[data.Timer_TextAlignment],
        Text_Alignment = Turbine.UI.ContentAlignment[data.Text_TextAlignment],

        Invisible_While_Timer_Active = data.Invisible_While_Timer_Active,
        Invisible_While_Timer_Inactive = data.Invisible_While_Timer_Inactive,

        Hide_Text_When_Inactive = data.Hide_Text_When_Inactive,
        Show_Base_Text_When_Inactive = data.Show_Base_Text_When_Inactive,

        Show_Duration = data.Show_Duration,
		Show_Text = data.Show_Text,
        Text = data.Text,
		Text_X_Offset = data.Text_X_Offset,
		Text_Y_Offset = data.Text_Y_Offset,

		Duration_X_Offset = data.Duration_X_Offset,
		Duration_Y_Offset = data.Duration_Y_Offset,

		Alignment = data["Alignment"],
        Width = data.Width,
        Height = data.Height,
        X = data.X,
        Y = data.Y,

        Vertical = data.Vertical,
        Ticks = data.Ticks,
        Bar_Direction = data.Bar_Direction,

        Do_Not_Update = data.Do_Not_Update,
        VisibitilityThreshold = data.Visibitility_Threshold,
    }


	local valuebar = ValueBar(parent, layoutTable)

    if self.groupMode then
        valuebar:SetPosition(data["X"], data["Y"])
        valuebar.moveBox = MoveBox(valuebar, data["Width"] + 2 * data["BorderSize"], data["Height"] + 2 * data["BorderSize"], data["NickName"] or "valuebar")
        valuebar.savePosition = function()
            data["X"], data["Y"] = valuebar:GetPosition()
        end
    else
        valuebar:SetPosition(self:ContentAlignement(data["Alignment"], data["Width"] + 2 * data["BorderSize"]) + (data.OffsetX or 0), self.currentHeight)
    end

    self.lookup[data.NickName or ""] = valuebar

    valuebar.TriggerFunction = function(triggerValue, triggerArgs, eventArgs)
        for key, value in pairs(data["Triggers"]) do
            if value.Trigger_Code == triggerValue then
                valuebar:Trigger(triggerArgs, eventArgs)
            end
        end
    end

    valuebar.AddTriggerFunction = function(tableName, value)
        local addTrigger = false
        if not valuebar[tableName] then
            valuebar[tableName] = {}
            addTrigger = true
        end

        if valuebar[tableName][value.Trigger_Code] then
            table.insert(valuebar[tableName][value.Trigger_Code], value)
        else
            valuebar[tableName][value.Trigger_Code] = {}
            table.insert(valuebar[tableName][value.Trigger_Code], value)
        end

        return addTrigger
    end

    if tablelength(data["Triggers"]) > 0 then
        for _, value in pairs(data["Triggers"]) do
            if value.Trigger_Type == "EventTrigger" then
                if valuebar.AddTriggerFunction("EventTriggerTable", value) then
                    table.insert(self.eventTriggerCallbacks, AddCallback(self, "EventTrigger", 
                    function(event)
                        if valuebar["EventTriggerTable"][event] then
                            valuebar.TriggerFunction(event, self.args[event], value)    
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "EffectAdded" then
                if valuebar.AddTriggerFunction("EffectAddedTable", value) then
                    table.insert(self.effectAddedCallbacks, AddCallback(PlayerEffectsHandler, "EffectAdded", 
                    function(effect)
                        if valuebar["EffectAddedTable"][effect:GetName()] then
                            for k, v in pairs(valuebar["EffectAddedTable"][effect:GetName()]) do
                                valuebar.TriggerFunction(effect:GetName(), {["Time"] = effect:GetDuration(), ["StartTime"] = effect:GetStartTime()}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "EffectAdded_Target" then
                if valuebar.AddTriggerFunction("EffectAddedTargetTable", value) then
                    table.insert(self.effectTargetAddedCallbacks, TargetHandlerObject:AddEffectCallback( 
                    function(args)
                        if valuebar["EffectAddedTargetTable"][args.Name] then
                            for k, v in pairs(valuebar["EffectAddedTargetTable"][args.Name]) do
                                valuebar.TriggerFunction(args.Name, {["Time"] = args.Duration, ["StartTime"] = args.StartTime}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "EffectAdded_Group" then
                if valuebar.AddTriggerFunction("EffectAddedGroupTable", value) then
                    table.insert(self.effectAddedGroupCallbacks, AddCallback(GroupEffectsHandler, "EffectAdded", 
                    function(args)
                        if valuebar["EffectAddedGroupTable"][args.Name] then
                            for k, v in pairs(valuebar["EffectAddedGroupTable"][args.Name]) do
                                valuebar.TriggerFunction(args.Name, {["Time"] = args.effect.Duration, ["StartTime"] = args.effect.StartTime}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "EffectRemoved" then
                if valuebar.AddTriggerFunction("EffectRemovedTable", value) then
                    table.insert(self.effectRemovedCallbacks, AddCallback(PlayerEffectsHandler, "EffectRemoved", 
                    function(effect)
                        if valuebar["EffectRemovedTable"][effect:GetName()] then
                            for k, v in pairs(valuebar["EffectRemovedTable"][effect:GetName()]) do
                                valuebar.TriggerFunction(effect:GetName(), {["Time"] = effect:GetDuration(), ["StartTime"] = effect:GetStartTime()}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "Skills" then
                self.skills = true
                for key, value in pairs(data["Skills"]) do
                    if SkillEventHandler:SkillExists(key) then
                        -- Add the callback for this skill slot
                        if not(self.skillCallbacks[key]) then self.skillCallbacks[key] = {} end
                        table.insert(self.skillCallbacks[key], AddCallback(SkillEventHandler, key, function(skill) 
                            if type(value) == "table" then
                                local time = value.Countdown
                                local maxTime = value.Countdown
                                if time == true then
                                    time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                                    maxTime = skill:GetCooldown()
                                end
                                if (skill:GetResetTime() - Turbine.Engine.GetGameTime()) + 1 <= maxTime then
                                    return
                                end
                                valuebar.bar:SetMaxValue(maxTime)
                                valuebar.bar:SetCurrentValue(time)
                            else
                                local time = value
                                local maxTime = value
                                if time == true then
                                    time = skill:GetResetTime() - Turbine.Engine.GetGameTime()
                                    maxTime = skill:GetCooldown()
                                end
                                if time + 1 <= maxTime then
                                    return
                                end
                                valuebar.bar:SetMaxValue(maxTime)
                                valuebar.bar:SetCurrentValue(time)
                            end
                        end))
                    end
                end
            end
        
            if value.Trigger_Type == "EnemyHits" then
                if valuebar.AddTriggerFunction("EnemyHitsTable", value) then
                    table.insert(self.enemyChatCallback, AddCallback(ChatEventHandler, "EnemyHits", 
                    function(args)
                        if args.Failed and not value.Also_Trigger_On_Resist then return end
                        if not args.Failed and value.Only_Trigger_On_Resist then return end
                        if valuebar["EnemyHitsTable"][args.SkillName] then
                            for k, v in pairs(valuebar["EnemyHitsTable"][args.SkillName]) do
                                valuebar.TriggerFunction(args.SkillName, {}, v)    
                            end
                        end
                    end))
                end        
            end
        
            if value.Trigger_Type == "PlayerHits" then
                if valuebar.AddTriggerFunction("PlayerHitsTable", value) then
                    table.insert(self.playerChatCallback, AddCallback(ChatEventHandler, "PlayerHits", 
                    function(args)
                        if valuebar["PlayerHitsTable"][args.SkillName] then
                            for k, v in pairs(valuebar["PlayerHitsTable"][args.SkillName]) do
                                valuebar.TriggerFunction(args.SkillName, {}, v)    
                            end
                        end
                    end))
                end 
            end
        
            if value.Trigger_Type == "PlayerHeal" then
                if valuebar.AddTriggerFunction("PlayerHealsTable", value) then
                    table.insert(self.playerHealCallbacks, AddCallback(ChatEventHandler, "PlayerHeal", 
                    function(args)
                        if valuebar["PlayerHealsTable"][args.SkillName] then
                            for k, v in pairs(valuebar["PlayerHealsTable"][args.SkillName]) do
                                valuebar.TriggerFunction(args.SkillName, {}, v)    
                            end
                        end
                    end))
                end 
            end
        
            if value.Trigger_Type == "PlayerBenefit" then
                if valuebar.AddTriggerFunction("PlayerBenefitTable", value) then
                    table.insert(self.playerBenefitCallbacks, AddCallback(ChatEventHandler, "PlayerBenefit", 
                    function(args)
                        if valuebar["PlayerBenefitTable"][args.SkillName] then
                            for k, v in pairs(valuebar["PlayerBenefitTable"][args.SkillName]) do
                                valuebar.TriggerFunction(args.SkillName, {}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "StandardChat" then
                if valuebar.AddTriggerFunction("StandardChatTable", value) then
                    table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Standard", 
                    function(message)
                        if valuebar["StandardChatTable"][message] then
                            for k, v in pairs(valuebar["StandardChatTable"][message]) do
                                valuebar.TriggerFunction(message, {}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "PartyChat" then
                if valuebar.AddTriggerFunction("PartyTable", value) then
                    table.insert(self.standardChatCallback, AddCallback(ChatEventHandler, "Party", 
                    function(message)
                        if valuebar["PartyTable"][message] then
                            for k, v in pairs(valuebar["PartyTable"][message]) do
                                valuebar.TriggerFunction(message, {}, v)    
                            end
                        end
                    end))
                end
            end
        
            if value.Trigger_Type == "SayChat" then
                if valuebar.AddTriggerFunction("SayTable", value) then
                    table.insert(self.sayChatCallback, AddCallback(ChatEventHandler, "SayLine", 
                    function(message)
                        if valuebar["SayTable"][message] then
                            for k, v in pairs(valuebar["SayTable"][message]) do
                                valuebar.TriggerFunction(message, {}, v)    
                            end
                        end
                    end))
                end
            end
        end
        
    end
    
    
    self.currentHeight = self.currentHeight + data["Height"] + 2 * data["BorderSize"]
    self.heightSize = self.heightSize + data["Height"] + 2 * data["BorderSize"]
    table.insert(self.tools, valuebar)
end