function GenerateValuebar2Options(valuebar, optionsHeight, width)
    valuebar.TriggerBox = OptionsControl(valuebar.optionsControl, width - 20, optionsHeight, {
        [1] = {
            ["Type"] = "Dropdown",
            ["Text"] = "Trigger Type",
            ["Variable"] = "Trigger_Type",
            ["Items"] = {
                "EffectAdded",
                "EffectAdded_Group",
                "EffectAdded_Target",
                "EffectRemoved",
                "EnemyHits",
                "EventTriggers",
                "PartyChat",
                "PlayerBenefit",
                "PlayerHeal",
                "PlayerHits",
                "SayChat",
                "Skills",
                "StandardChat",
            }
        },
        [2] = {
            ["Type"] = "Label String Textbox",
            ["Text"] = "Event",
            ["Variable"] = "Trigger_Event",
        }
    })

    optionsHeight = optionsHeight + valuebar.TriggerBox:GetHeight() + 5

    valuebar.DurationBox = OptionsControl(valuebar.optionsControl, width - 20, optionsHeight, {
        [1] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "Start Timer",
            ["Variable"] = "Timer",
        },
        [2] = {
            ["Type"] = "Label Number Textbox",
            ["Text"] = "Timer Duration",
            ["Variable"] = "Timer_Duration",
        },
        [3] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "From Trigger",
            ["Variable"] = "Special_Trigger",
        },
    })

    optionsHeight = optionsHeight + valuebar.DurationBox:GetHeight() + 5

    valuebar.ChangeText = OptionsControl(valuebar.optionsControl, width - 20, optionsHeight, {
        [1] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "Change Text",
            ["Variable"] = "Change_Text",
        },
        [2] = {
            ["Type"] = "Label String Textbox",
            ["Text"] = "Text",
            ["Variable"] = "New_Text",
        }
    })

    optionsHeight = optionsHeight + valuebar.ChangeText:GetHeight() + 5

    return optionsHeight
end

function AddButtonFunctionValuebar2(valuebar, width, triggerType, UIType, func)
    local trigger_result = valuebar.TriggerBox:GetValues()
    local timer_result = valuebar.DurationBox:GetValues()
    local change_text_result = valuebar.ChangeText:GetValues()

    if trigger_result.Trigger_Type == "Trigger Type" then
        ThrowException("Failed to add trigger. Trying to add trigger without trigger type.")
        return     
    end

    local triggerEntry = {
        ["Trigger_Code"] = trigger_result.Trigger_Event,
        ["Trigger_Type"] = trigger_result.Trigger_Type,

        ["Timer"] = timer_result.Timer,
        ["Timer_Duration"] = timer_result.Timer_Duration,
        ["Special_Trigger"] = timer_result.Special_Trigger,

        ["Change_Text"] = change_text_result.Change_Text,
        ["New_Text"] = change_text_result.New_Text,
    }

    if func(Turbine.Engine.GetGameTime(), triggerEntry) then
        return
    end

    valuebar.listbox:AddItem(CreateTriggerListBoxOptionsListboxItemValuebar2(Turbine.Engine.GetGameTime(), triggerEntry, width - 10, triggerType, UIType, func, valuebar.listbox))
    valuebar.listbox:SetVisible(true)
end

function CreateTriggerListBoxOptionsListboxItemValuebar2(key, value, width, triggerType, UIType, func, listbox)
    local listBoxItem = Turbine.UI.Control()
    listBoxItem:SetMouseVisible(false)
    listBoxItem:SetSize(width, 20)
    listBoxItem:SetBackColor(Turbine.UI.Color.Black)

    local innerWidth = width - 2

    local optionsHeight = 0

    listBoxItem.InnerBox = Turbine.UI.Control()
    listBoxItem.InnerBox:SetParent(listBoxItem)
    listBoxItem.InnerBox:SetPosition(1, 1)
    listBoxItem.InnerBox:SetSize(innerWidth, 18)
    listBoxItem.InnerBox:SetBackColor(Turbine.UI.Color.DarkSlateBlue)
    
    listBoxItem.label = Turbine.UI.Label()
    listBoxItem.label:SetParent(listBoxItem.InnerBox)
	listBoxItem.label:SetSize(innerWidth, 25)
	listBoxItem.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold22)
	listBoxItem.label:SetPosition(0, optionsHeight)
    listBoxItem.label:SetText(tostring(value.Trigger_Type) .. ": " .. tostring(value.Trigger_Code))
    listBoxItem.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    listBoxItem.label:SetFontStyle(Turbine.UI.FontStyle.Outline)
    listBoxItem.label:SetOutlineColor(Turbine.UI.Color.Black)
    listBoxItem.label:SetBackColor(Turbine.UI.Color(0.2,0.15,0.53))

    
    listBoxItem.RemovalButton = Turbine.UI.Lotro.Button()
    listBoxItem.RemovalButton:SetParent(listBoxItem.InnerBox)
    listBoxItem.RemovalButton:SetSize(100, 20)
    listBoxItem.RemovalButton:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    listBoxItem.RemovalButton:SetMouseVisible(false)
    listBoxItem.RemovalButton:SetPosition(width - 110, optionsHeight + 2)
    listBoxItem.RemovalButton:SetText("Remove")

    optionsHeight = optionsHeight + listBoxItem.RemovalButton:GetHeight() + 5

    listBoxItem.RemovalButton.Click = function()
        func(key, nil)
        listbox:RemoveItem(listBoxItem)
    end

    listBoxItem.options = {}

    listBoxItem.DurationBox = OptionsControl(listBoxItem.InnerBox, width - 20, optionsHeight, {
        [1] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "Start Timer",
            ["Variable"] = "Timer",
            ["Value"] = value.Timer
        },
        [2] = {
            ["Type"] = "Label Number Textbox",
            ["Text"] = "Timer Duration",
            ["Variable"] = "Timer_Duration",
            ["Value"] = value.Timer_Duration
        },
        [3] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "From Trigger",
            ["Variable"] = "Special_Trigger",
            ["Value"] = value.Special_Trigger
        },
    })

    listBoxItem.DurationBox.ValueChanged = function()
        listBoxItem.ChangeFunction()
    end

    optionsHeight = optionsHeight + listBoxItem.DurationBox:GetHeight() + 5

    listBoxItem.ChangeText = OptionsControl(listBoxItem.InnerBox, width - 20, optionsHeight, {
        [1] = {
            ["Type"] = "Enable Checkbox",
            ["Text"] = "Change Text",
            ["Variable"] = "Change_Text",
            ["Value"] = value.Change_Text
        },
        [2] = {
            ["Type"] = "Label String Textbox",
            ["Text"] = "Text",
            ["Variable"] = "New_Text",
            ["Value"] = value.New_Text
        }
    })

    listBoxItem.ChangeText.ValueChanged = function()
        listBoxItem.ChangeFunction()
    end

    optionsHeight = optionsHeight + listBoxItem.ChangeText:GetHeight() + 5

    listBoxItem.InnerBox:SetSize(innerWidth, optionsHeight)
    listBoxItem:SetSize(width, optionsHeight + 2)

    listBoxItem.ChangeFunction = function()
        func(key, nil)

        local timer_result = listBoxItem.DurationBox:GetValues()
        local change_text_result = listBoxItem.ChangeText:GetValues()

        local triggerEntry = {
            ["Trigger_Code"] = value.Trigger_Code,
            ["Trigger_Type"] = value.Trigger_Type,

            ["Timer"] = timer_result.Timer,
            ["Timer_Duration"] = timer_result.Timer_Duration,
            ["Special_Trigger"] = timer_result.Special_Trigger,

            ["Change_Text"] = change_text_result.Change_Text,
            ["New_Text"] = change_text_result.New_Text,
        }
        func(key, triggerEntry) 
    end

    return listBoxItem
end