function CreateTriggerListBoxOptions(width, triggerType, listboxItems, UIType, func, optionPage)
    local headerNode = Turbine.UI.TreeNode()
    headerNode:SetMouseVisible(false)
    headerNode:SetSize(width, 20)

    -- The header label with the trigger name
    headerNode.label = Turbine.UI.Label()
    headerNode.label:SetParent(headerNode)
    headerNode.label:SetSize(width, 20)
    headerNode.label:SetFont(Turbine.UI.Lotro.Font.BookAntiquaBold19)
    headerNode.label:SetPosition(0, 0)
    headerNode.label:SetText(string.format("%s (%s)", triggerType, tostring(tablelength(listboxItems))))
    headerNode.label:SetBackColor(Turbine.UI.Color.Black)

    local optionsTreeNode = headerNode:GetChildNodes()

    local valuebar = Turbine.UI.TreeNode()
    valuebar:SetPosition(0, 0)
    valuebar:SetMouseVisible(false)
   --valuebar:SetBackColor(GetRandomColour())

   local optionsHeight = 10
   local listboxHeight = 700

    valuebar.optionsControl = Turbine.UI.Control()
	valuebar.optionsControl:SetParent(valuebar)
    valuebar.optionsControl:SetPosition(0, 0)
    valuebar.optionsControl:SetBackColor(Turbine.UI.Color.DarkSlateGray)

    valuebar.options = {}

    if UIType == "TimerIcon2" then
        optionsHeight = PrimeUIElementOptions.GenerateTimerIcon2Options(valuebar, optionsHeight, width)
    elseif UIType == "ValueBar2" then
        optionsHeight = PrimeUIElementOptions.GenerateValuebar2Options(valuebar, optionsHeight, width)
    elseif UIType == "ResourceBar2" then
        optionsHeight = PrimeUIElementOptions.GenerateResourebarOptions(valuebar, optionsHeight, width)
    end
    

    valuebar.addTriggerEntryButton = Turbine.UI.Lotro.Button()
    valuebar.addTriggerEntryButton:SetParent(valuebar)
    valuebar.addTriggerEntryButton:SetSize(100, 20)
    valuebar.addTriggerEntryButton:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    valuebar.addTriggerEntryButton:SetPosition(width - 120, optionsHeight)
    valuebar.addTriggerEntryButton:SetText("Add")

    optionsHeight = optionsHeight + 20

    -- Set options control to match all its children
    valuebar.optionsControl:SetSize(width, optionsHeight)

    -- Add listbox for items
    valuebar.listbox = Turbine.UI.ListBox()
    valuebar.listbox:SetParent(valuebar)
    valuebar.listbox:SetSize(width - 10, listboxHeight)
    valuebar.listbox:SetPosition(0, valuebar.optionsControl:GetHeight())
    valuebar.listbox:SetVisible(tablelength(listboxItems) > 0)

    valuebar.Scrollbar = Turbine.UI.Lotro.ScrollBar()
    valuebar.Scrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)
    valuebar.Scrollbar:SetParent(valuebar)
    valuebar.Scrollbar:SetPosition(width - 10, valuebar.optionsControl:GetHeight())
    valuebar.Scrollbar:SetSize(10, listboxHeight)
    valuebar.Scrollbar:SetBackColor(Turbine.UI.Color.DarkSlateGray)
    valuebar.listbox:SetVerticalScrollBar(valuebar.Scrollbar)

    for key, value in spairs(listboxItems, function(t, a, b) return tostring(a) < tostring(b) end) do
        if UIType == "TimerIcon2" then
            valuebar.listbox:AddItem(PrimeUIElementOptions.CreateTriggerListBoxOptionsListboxItemTimerIcon2(key, value, width - 10, triggerType, UIType, func, valuebar.listbox))
        elseif UIType == "ValueBar2" then
            valuebar.listbox:AddItem(PrimeUIElementOptions.CreateTriggerListBoxOptionsListboxItemValuebar2(key, value, width - 10, triggerType, UIType, func, valuebar.listbox))
        elseif UIType == "ResourceBar2" then
            valuebar.listbox:AddItem(PrimeUIElementOptions.CreateTriggerListBoxOptionsListboxItemResourcebar(key, value, width - 10, triggerType, UIType, func, valuebar.listbox))
        end
    end

    valuebar:SetSize(width, valuebar.listbox:GetHeight() + valuebar.optionsControl:GetHeight())

    valuebar.addTriggerEntryButton.Click = function()
        if UIType == "TimerIcon2" then
            PrimeUIElementOptions.AddButtonFunctionTimerIcon2(valuebar, width, triggerType, UIType, func)
        elseif UIType == "ValueBar2" then
            PrimeUIElementOptions.AddButtonFunctionValuebar2(valuebar, width, triggerType, UIType, func)
        elseif UIType == "ResourceBar2" then
            PrimeUIElementOptions.AddButtonFunctionResourcebar(valuebar, width, triggerType, UIType, func)
        end
    end

    optionsTreeNode:Add(valuebar)

    return headerNode
end

OptionsControl = class(Turbine.UI.Control)
function OptionsControl:Constructor(parent, width, y, args)
    Turbine.UI.Control.Constructor(self)

    self.width = width
    self.height = 0
    self.borderSize = 2

    self:SetParent(parent)
    self:SetMouseVisible(false)
    self:SetPosition(10, y)
    self:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    self:SetBackColor(Turbine.UI.Color.Black)

    self.currentBackground = Turbine.UI.Control()
    self.currentBackground:SetParent(self)
    self.currentBackground:SetMouseVisible(false)

    self.currentBackground:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)
    self.currentBackground:SetBackColor(Turbine.UI.Color.DarkGray)

    self.UI = {}

    local optionsHeight = 5

    for key, value in ipairs(args) do
        if value.Type == "Enable Checkbox" then
            self.UI[value.Variable] = Turbine.UI.Lotro.CheckBox()
            self.UI[value.Variable]:SetParent(self.currentBackground)
            self.UI[value.Variable]:SetSize(300, 20)
            self.UI[value.Variable]:SetPosition(5, optionsHeight)
            self.UI[value.Variable]:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
            self.UI[value.Variable]:SetForeColor(Turbine.UI.Color.Black)
            self.UI[value.Variable]:SetText(value.Text)
            self.UI[value.Variable]:SetChecked(value.Value)
            self.UI[value.Variable].CheckedChanged = function ()
                self:ValueChanged()
            end
            self.UI[value.Variable].GetValue = function()
                return self.UI[value.Variable]:IsChecked()
            end

            optionsHeight = optionsHeight + 25
        elseif value.Type == "Label Number Textbox" or value.Type == "Label String Textbox" then
            self.UI[value.Variable] = LabelTextboxControl(self.currentBackground, width - 20, optionsHeight, value, value.Type == "Label Number Textbox")
            self.UI[value.Variable].ValueChanged = function()
                self:ValueChanged()
            end
            optionsHeight = optionsHeight + 25
        elseif value.Type == "Dropdown" then
            self.UI[value.Variable] = DropdownControl(self.currentBackground, width - 20, optionsHeight, value)
            self.UI[value.Variable].ValueChanged = function()
                self:ValueChanged()
            end
            optionsHeight = optionsHeight + 25
        end
    end
    self.height = optionsHeight
    self:Format()
end

function OptionsControl:Format()
    self:SetSize(self.width, self.height + 2 * self.borderSize)
    self.currentBackground:SetPosition(self.borderSize, self.borderSize)
    self.currentBackground:SetSize(self.width - 2 * self.borderSize, self.height)
end

function OptionsControl:ValueChanged()

end

function OptionsControl:GetValues()
    local valueTable = {}
    for key, value in pairs(self.UI) do
        valueTable[key] = value:GetValue()
    end
    return valueTable
end

LabelTextboxControl = class(Turbine.UI.Control)
function LabelTextboxControl:Constructor(parent, width, y, args, number)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetMouseVisible(false)
    self:SetPosition(0, y)
    self:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)

    self.width = width
    self.number = number

    self.label = Turbine.UI.Label()
    self.label:SetParent(self)
    self.label:SetPosition(5, 0)
    self.label:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.label:SetForeColor(Turbine.UI.Color.Black)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetText(args.Text)

    self.textbox = Turbine.UI.Lotro.TextBox()
    self.textbox:SetParent(self)
    self.textbox:SetForeColor(Turbine.UI.Color.White)
    self.textbox:SetFont(Turbine.UI.Lotro.Font.VerdanaBold16)
    self.textbox:SetText(args.Value and tostring(args.Value))

    self.textbox.TextChanged = function ()
        self:ValueChanged()
    end

    self:Format()
end

function LabelTextboxControl:ValueChanged()

end

function LabelTextboxControl:Format()
    self:SetSize(self.width, 20)
    self.label:SetSize(self.width * .3, 20)
    self.textbox:SetPosition(self.width * .3, 0)
    self.textbox:SetSize(self.width * .7, 20)
end

function LabelTextboxControl:GetValue()
    if self.number then
        return tonumber(tostring(self.textbox:GetText()))
    else
        return tostring(self.textbox:GetText())
    end
end

function LabelTextboxControl:SetValue(value)
    self.textbox:SetText(tostring(value))
end

DropdownControl = class(Turbine.UI.Control)
function DropdownControl:Constructor(parent, width, y, args)
    Turbine.UI.Control.Constructor(self)
    self:SetParent(parent)
    self:SetMouseVisible(false)
    self:SetPosition(0, y)
    self:SetBackColorBlendMode(Turbine.UI.BlendMode.Overlay)

    self.width = width

    self.label = Turbine.UI.Lotro.Button()
    self.label:SetParent(self)
    self.label:SetPosition(5, 0)
    self.label:SetText(args.Text)

    self.label.TextChanged = function ()
        self:ValueChanged()
    end

    self.menu = Turbine.UI.ContextMenu()
    self.menuItems = self.menu:GetItems()

    local sortedKeys2 = {}
    for key, value in pairs(args.Items) do table.insert( sortedKeys2, value ) end
    table.sort( sortedKeys2 )
    
    for key, name in pairs(sortedKeys2) do
        local menuItem = Turbine.UI.MenuItem( name )

        menuItem.Click = function()
            self.class = name
            self.label:SetText(name)
        end
        
        self.menuItems:Add(menuItem)
    end

    self.label.Click = function()
        self.menu:ShowMenu()
    end

    self:Format()
end

function DropdownControl:ValueChanged()

end

function DropdownControl:Format()
    self:SetSize(self.width, 20)
    self.label:SetSize(self.width * .3, 20)
end

function DropdownControl:GetValue()
    return tostring(self.label:GetText())
end

function DropdownControl:SetValue(value)
    self.label:SetText(tostring(value))
end