ResourceBar = class( Turbine.UI.Control )
function ResourceBar:Constructor(parent, data)
    Turbine.UI.Control.Constructor(self)

    self.playerClass = Turbine.Gameplay.LocalPlayer.GetInstance():GetClass()

    -- Config sizes
	self.borderSize = data.BorderSize or 0
    self.width = data.Width or 1
    self.height = data.Height or 1
	self.barWidth = self.width - 2 * self.borderSize
	self.barHeight = self.height - 2 * self.borderSize
    self.tickWidth = data.TickWidth or 0
    self.boxOffset = data.Box_Offset or 0
    self.boxBorderSize = data.Box_BorderSize or 0
    self.textHeight = data.Text_Height or 20
    self.textOffsetX = data.Text_Offset_X or 0
    self.textOffsetY = data.Text_Offset_Y or 0

    -- Config Colours
    self.baseBoxBackgroundColour = data.Base_Box_BackgroundColour or Turbine.UI.Color.Black
    self.baseBoxColour = data.Base_Box_Colour or Turbine.UI.Color.Black
    self.barBackgroundColour = data.Bar_Background_Colour or Turbine.UI.Color.Black
    self.barColour = data.Bar_Colour or Turbine.UI.Color.Black
    self.borderColour = data.Border_Colour or Turbine.UI.Color.Black

    --Config misc
    self.vertical = data.Vertical
    self.ticks = data.Ticks or {}

    -- Process info
    self.visibitilityThreshold = data.VisibitilityThreshold or 999999
    self.stopAutoUpdate = data.Do_Not_Update
    self.barDirection = data.Bar_Direction or 1

	-- Text info
    self.textFont = data.Text_Font or Turbine.UI.Lotro.Font.FixedSys15
    self.fonttextColour = data.Text_Font_Colour or Turbine.UI.Color(1,1,1)

	self.durationAlignment = data.Timer_Alignment
    self.textAlignment = data.Text_Alignment

	self.text = data.Text

    self.showText = data.Show_Text
    self.showDuration = data.Show_Duration

	self.textXOfsset = data.Text_X_Offset
	self.textYOfsset = data.Text_Y_Offset
	self.durationXOfsset = data.Duration_X_Offset
	self.durationYOfsset = data.Duration_Y_Offset

    self.invistWhileActive = data.Invisible_While_Timer_Active
    self.invisWhileInactive = data.Invisible_While_Timer_Inactive

    
    self.colourRulesForeground = data.Additional_Colour_Rules_Foreground or {}
    self.colourRulesBackground = data.Additional_Colour_Rules_Background or {}

    self.colourOptionsForeground = {}
    for key, value in spairs(self.colourRulesForeground, function(t, a, b) return t[a].Box_Number < t[b].Box_Number end) do
        self.colourOptionsForeground[value.Box_Number] = {["Colour"] = Turbine.UI.Color(value.Red/255, value.Green/255, value.Blue/255), ["Upper"] = value.Box_Upper}
    end

    self.colourOptionsBackground = {}
    for key, value in spairs(self.colourRulesBackground, function(t, a, b) return t[a].Box_Number < t[b].Box_Number end) do
        self.colourOptionsBackground[value.Box_Number] = {["Colour"] = Turbine.UI.Color(value.Red/255, value.Green/255, value.Blue/255), ["Upper"] = value.Box_Upper}
    end

    


    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetMouseVisible(false)
	self:SetBackColor(self.borderColour)
    self:SetVisible(true)

    self.MainBarBackground = Turbine.UI.Control()
	self.MainBarBackground:SetParent(self)
	self.MainBarBackground:SetMouseVisible(false)
	self.MainBarBackground:SetPosition(self.borderSize, self.borderSize)
	self.MainBarBackground:SetSize(self.barWidth, self.barHeight)
	self.MainBarBackground:SetBackColor(self.barBackgroundColour)

    self.textWindow = Turbine.UI.Window()
	self.textWindow:SetParent(self)
    self.textWindow:SetVisible(true)
	self.textWindow:SetMouseVisible(false)
	self.textWindow:SetPosition(self.textOffsetX, self.barHeight / 2 - self.textHeight / 2 + self.textOffsetY )
	self.textWindow:SetSize(self.barWidth, self.textHeight)

    self.textLabel = Turbine.UI.Label()
    self.textLabel:SetParent(self.textWindow)
	self.textLabel:SetMouseVisible(false)
	self.textLabel:SetPosition(0, 0)
	self.textLabel:SetSize(self.textWindow:GetWidth(), self.textWindow:GetHeight())
    self.textLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter)
    self.textLabel:SetFont(self.textFont)
    self.textLabel:SetForeColor(self.fonttextColour)
    self.textLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.textLabel:SetOutlineColor(Turbine.UI.Color.Black)
    self.textLabel:SetVisible(self.showText)

    self.SizeChanged = function()
        self.textWindow:SetSize(self:GetWidth(), self.textHeight)
        self.textLabel:SetSize(self.textWindow:GetWidth(), self.textWindow:GetHeight())
    end

    self.boxes = {}

    if self.playerClass == Turbine.Gameplay.Class.Champion then
        self.boxCount = 5
    elseif self.playerClass == Turbine.Gameplay.Class.Hunter then
        self.boxCount = 9
    elseif self.playerClass == Turbine.Gameplay.Class.RuneKeeper then
        self.boxCount = 18
    elseif self.playerClass == (Turbine.Gameplay.Class.Mariner or Turbine.Gameplay.Class.Corsair or Turbine.Gameplay.Class.Chicken) then
        self.boxCount = 49
    else
        self.boxCount = 1
    end

    local newWidth = math.floor((self.barWidth - self.boxOffset * (self.boxCount - 1))/self.boxCount)

    for i = 1, self.boxCount do
        local offset = i == 1 and 0 or math.floor((i - 1) * newWidth) + self.boxOffset * (i - 1)
        self.boxes[i] = self:CreateBox(newWidth, self.barHeight, offset + self.borderSize, self.borderSize, i)
    end
    self:SetWidth(newWidth * self.boxCount + self.boxOffset * (self.boxCount - 1) + 2 * self.borderSize)
end

function ResourceBar:SetAttunement(value)
    if value < 10 then
        for i = 1, self.boxCount do
            if i < value then
                self.boxes[i]:TurnOff()
            else
                if i >= 10 then
                    self.boxes[i]:TurnOff()
                else
                    self.boxes[i]:TurnOn()
                end
            end
        end
    elseif value > 10 then
        local adjValue = value - 1
        for i = 1, self.boxCount do
            if i <= 9 then
                self.boxes[i]:TurnOff()
            else
                if i <= adjValue then
                    self.boxes[i]:TurnOn()
                else
                    self.boxes[i]:TurnOff()
                end
            end
        end
    else
        for i = 1, self.boxCount do
            self.boxes[i]:TurnOff()
        end
    end

    self.textLabel:SetText("DD")
end

function ResourceBar:SetBalance(value)
    local halfway = self.boxCount / 2
    if value < halfway then
        for i = 1, self.boxCount do
            if i < value then
                self.boxes[i]:TurnOff()
            else
                if i >= halfway then
                    self.boxes[i]:TurnOff()
                else
                    self.boxes[i]:TurnOn()
                end
                
            end
        end
    elseif value > halfway then
        for i = 1, self.boxCount do
            if i < halfway then
                self.boxes[i]:TurnOff()
            else
                if i >= value then
                    self.boxes[i]:TurnOff()
                else
                    self.boxes[i]:TurnOn()
                end
                
            end
        end
    else
        for i = 1, self.boxCount do
            self.boxes[i]:TurnOff()
        end
    end

    self.textLabel:SetText(value)
end

function ResourceBar:SetValue(value)
    for i = 1, self.boxCount do
        if i <= value then
            self.boxes[i]:TurnOn()
        else
            self.boxes[i]:TurnOff()
        end
    end
    self.textLabel:SetText(value)
end


function ResourceBar:CreateBox(width, height, x, y, id)

    local box = Turbine.UI.Control()
    box:SetParent(self)
    box:SetSize(width, height)
    box:SetPosition(x, y)
    box.Colour = self.baseBoxBackgroundColour

    for key, value in pairs(self.colourOptionsBackground) do
        if key <= id and id < value.Upper then
            box.Colour = value["Colour"]
        end
    end

    box:SetBackColor(box.Colour)

    local innerbox  = Turbine.UI.Control()
    innerbox:SetParent(box)
    innerbox:SetSize(width - 2 * self.boxBorderSize, height - 2 * self.boxBorderSize)
    innerbox:SetPosition(self.boxBorderSize, self.boxBorderSize)

    innerbox:SetVisible(false)
    innerbox.Colour = self.baseBoxColour

    for key, value in pairs(self.colourOptionsForeground) do
        if key <= id and id < value.Upper then
            innerbox.Colour = value["Colour"]
        end
    end

    innerbox:SetBackColor(innerbox.Colour)
    --[[
    local label = Turbine.UI.Label()
    label:SetParent(box)
    label:SetSize(width, height)
    label:SetText(id)
    --]]

    box.TurnOn = function()
        box:SetBackColor(box.Colour)
        innerbox:SetVisible(true)
    end

    box.TurnOff = function()
        innerbox:SetVisible(false)
    end

    return box
end

function ResourceBar:Unload()
end