ValueBar = class(Turbine.UI.Window)
function ValueBar:Constructor(parent, data)
    Turbine.UI.Window.Constructor(self)

    -- Config sizes
	self.borderSize = data.BorderSize or 0
    self.width = data.Width or 1
    self.height = data.Height or 1
	self.barWidth = self.width - 2 * self.borderSize
	self.barHeight = self.height - 2 * self.borderSize
    self.tickWidth = data.TickWidth or 0

    -- Config Colours
    self.tickColour = data.Tick_Colour or Turbine.UI.Color.Black
    self.barBackgroundColour = data.Bar_Background_Colour or Turbine.UI.Color.Black
    self.barColour = data.Bar_Colour or Turbine.UI.Color.Black
    self.borderColour = data.Border_Colour or Turbine.UI.Color.Black

    --Config misc
    self.vertical = data.Vertical
    self.ticks = data.Ticks or {}

    -- Process info
    self.visibitilityThreshold = data.VisibitilityThreshold or 999999
    self.stopAutoUpdate = data.Do_Not_Update
    self.barDirection = data.Bar_Direction or 1

	-- Text info
	self.timerFont = data.Timer_Font or Turbine.UI.Lotro.Font.FixedSys15
    self.textFont = data.Text_Font or Turbine.UI.Lotro.Font.FixedSys15
    self.fontTimeColour = data.Timer_Font_Colour or Turbine.UI.Color(1,1,1)
    self.fonttextColour = data.Text_Font_Colour or Turbine.UI.Color(1,1,1)
    
    self.showTextWhenInactive = data.Hide_Text_When_Inactive
    self.showBaseText = data.Show_Base_Text_When_Inactive

	self.durationAlignment = data.Timer_Alignment
    self.textAlignment = data.Text_Alignment

	self.text = data.Text

    self.showText = data.Show_Text
    self.showDuration = data.Show_Duration

	self.textXOfsset = data.Text_X_Offset
	self.textYOfsset = data.Text_Y_Offset
	self.durationXOfsset = data.Duration_X_Offset
	self.durationYOfsset = data.Duration_Y_Offset

    self.invistWhileActive = data.Invisible_While_Timer_Active
    self.invisWhileInactive = data.Invisible_While_Timer_Inactive

    self.barAlignment = data.Bar_Direction == "Left" and 3 or 
    data.Bar_Direction == "Middle" and 1 or
    data.Bar_Direction == "Right" and 2


    self:SetParent(parent)
    self:SetSize(self.width, self.height)
    self:SetMouseVisible(false)
	self:SetBackColor(self.borderColour)
    self:SetVisible(not(self.invisWhileInactive))

    self.widthChange = -999
    self.delta = 0
    self.NumberMethod = 1
    self.PercentageShown = false
    self.OnlyShowCurrent = true
    self.CurrentImmediate = 0
    self.CurrentValue = 0--Turbine.Animation.LerpValue()
    self.MaxValue = 0--Turbine.Animation.LerpValue()
    self.baseMax = 0--Turbine.Animation.LerpValue()
    
    -- The background bar that contains all other bars
	self.MainBarBackground = Turbine.UI.Control()
	self.MainBarBackground:SetParent(self)
	self.MainBarBackground:SetMouseVisible(false)
	self.MainBarBackground:SetPosition(self.borderSize, self.borderSize)
	self.MainBarBackground:SetSize(self.barWidth, self.barHeight)
	self.MainBarBackground:SetBackColor(self.barBackgroundColour)

	self.MainBar = Turbine.UI.Control()
	self.MainBar:SetParent(self.MainBarBackground)
	self.MainBar:SetMouseVisible(false)
	self.MainBar:SetSize(0, 0)
	self.MainBar:SetBackColor(self.barColour)

    -- Text
    self.TextWindow = Turbine.UI.Window()
    self.TextWindow:SetParent(self.MainBarBackground)
    self.TextWindow:SetSize(self.TextWindow:GetParent():GetWidth(), self.TextWindow:GetParent():GetHeight())
    self.TextWindow:SetPosition(self.textXOfsset, self.textYOfsset)
    self.TextWindow:SetVisible(true)

    self.TextLabel = Turbine.UI.Label()
    self.TextLabel:SetParent(self.TextWindow)
    self.TextLabel:SetSize(self.TextLabel:GetParent():GetWidth(), self.TextLabel:GetParent():GetHeight())
    self.TextLabel:SetMouseVisible(false)
    self.TextLabel:SetTextAlignment(self.textAlignment)
    self.TextLabel:SetFont(self.textFont)
    self.TextLabel:SetForeColor(self.fonttextColour)
    self.TextLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.TextLabel:SetOutlineColor(Turbine.UI.Color.Black)
    if self.showText then
        self.TextLabel:SetText(self.text)
    end

    -- Duration
    self.DurationWindow = Turbine.UI.Window()
    self.DurationWindow:SetParent(self.MainBarBackground)
    self.DurationWindow:SetSize(self.DurationWindow:GetParent():GetWidth(), self.DurationWindow:GetParent():GetHeight())
    self.DurationWindow:SetPosition(self.durationXOfsset, self.durationYOfsset)
    self.DurationWindow:SetVisible(true)

    self.DurationLabel = Turbine.UI.Label()
    self.DurationLabel:SetParent(self.DurationWindow)
    self.DurationLabel:SetSize(self.DurationLabel:GetParent():GetWidth(), self.DurationLabel:GetParent():GetHeight())
    self.DurationLabel:SetMouseVisible(false)
    self.DurationLabel:SetTextAlignment(self.durationAlignment)
    self.DurationLabel:SetFont(self.timerFont)
    self.DurationLabel:SetForeColor(self.fontTimeColour)
    self.DurationLabel:SetFontStyle(Turbine.UI.FontStyle.Outline)
    self.DurationLabel:SetOutlineColor(Turbine.UI.Color.Black)

	self.tickObjects = {}

	for key, value in pairs(self.ticks) do
		self.tickObjects[key] = self:GetTickObject(value)
	end

    self.ExecuteUpdate = function(delta)
        self.CurrentValue = self.CurrentValue - delta

        self:SetVisible(not self.invistWhileActive and self.CurrentValue < self.visibitilityThreshold)
    
        if self.CurrentValue < 0 then
			if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
			self.updateCallback = nil

            if self.showTextWhenInactive then
                self.TextLabel:SetVisible(false)
            end

            if self.showBaseText then
                self.TextLabel:SetText(self.text or "")
            end

            if self.invisWhileInactive then
                self:SetVisible(false)
            else
                self:SetVisible(true)
            end
        end

		self:AdjustBars()
    end
end

function ValueBar:GetTickObject(distance)
	local tickControl = Turbine.UI.Control()
	tickControl:SetParent(self.MainBarBackground)
	tickControl:SetMouseVisible(false)
	tickControl:SetPosition(self.MainBarBackground:GetWidth() * (distance / 100) - (self.tickWidth / 2), 0)
	tickControl:SetSize(self.tickWidth, self.tickWidth)
	tickControl:SetBackColor(self.barBackgroundColour)

	return tickControl
end

function ValueBar:SetValues(current, max)
	self.MaxValue = max
	self.CurrentValue = current

    self.TextLabel:SetVisible(true)
    if current > 0 then
        if self.invistWhileActive then
            self:SetVisible(false)
        else
            self:SetVisible(true)
        end
    end

    if not(self.updateCallback) and not(self.stopAutoUpdate) then self.updateCallback = AddCallback(Updater, "ExecuteUpdate", self.ExecuteUpdate) end
    if self.stopAutoUpdate then
        self:AdjustBars()
    end
end

function ValueBar:Trigger(triggerArgs, eventArgs)
    if eventArgs.Timer then
        local timer = eventArgs.Timer_Duration or 0
        if eventArgs.Special_Trigger and triggerArgs.Time then
            timer = triggerArgs.Time
        end

        self:SetValues(timer, timer)
    end

    if eventArgs.Change_Text then
        self.TextLabel:SetText(tostring(eventArgs.New_Text))
    end
end


function ValueBar:AdjustBars()

    if self.showDuration then
        self.DurationLabel:SetText(StringifyDuration(self.CurrentValue))
        if self.CurrentValue <= 0 then
            self.DurationLabel:SetVisible(false)
        else
            self.DurationLabel:SetVisible(true)
        end
    end

   local currentWidth = self.CurrentValue / self.MaxValue

    if self.vertical then
        currentWidth = currentWidth * self.barHeight
    else         
        currentWidth = currentWidth * self.barWidth
    end

	-- From left to right
    if self.barAlignment == 1 then
        local newWidth = math.ceil(currentWidth)

		if self.vertical then
			self.MainBar:SetSize(self.barWidth, newWidth)
			self.MainBar:SetTop(math.ceil( (self.barHeight - currentWidth) / 2))
		else
			self.MainBar:SetSize(newWidth, self.barHeight)
			self.MainBar:SetLeft(math.ceil( (self.width - currentWidth) / 2))
		end
	-- Middle
    elseif self.barAlignment == 2 then
        if self.vertical then
            self.MainBar:SetSize(self.barWidth, math.ceil(currentWidth))
            self.MainBar:SetTop(self.barHeight - math.ceil(currentWidth))
        else
            self.MainBar:SetSize(math.ceil(currentWidth), self.barHeight)
            self.MainBar:SetLeft(self.barWidth - math.ceil(currentWidth))
        end
	-- From right to left
    else
        if self.vertical then
            self.MainBar:SetSize(self.barWidth, math.ceil(currentWidth))
        else
            self.MainBar:SetSize(math.ceil(currentWidth), self.barHeight)
        end
    end
end

function ValueBar:Unload()

    if self.updateCallback then if self.updateCallback then RemoveCallback(Updater, "ExecuteUpdate", self.updateCallback) end end
    self.updateCallback = nil
    self.TextWindow:SetParent(nil)
    self:SetParent(nil)
end