Table = class(Turbine.UI.Window)
function Table:Constructor(parent, layoutTable, headerData, data, names)
    Turbine.UI.Window.Constructor(self)
    self:SetVisible(true)
    self:SetParent(parent)

    self.width = layoutTable.Width
    self.height = layoutTable.Height

    self:SetSize(self.width, self.height)

    self.columns = layoutTable.Columns or 2
    self.rows = layoutTable.Rows or 2

    self.lineColour = layoutTable.Line_Colour or Turbine.UI.Color.Black
    self.cellColour = layoutTable.Cell_Colour or Turbine.UI.Color.White

    self.cellWidth = layoutTable.Cell_Width or 200
    self.cellHeight = layoutTable.Cell_Height or 20

    self.lineWidth = layoutTable.Line_Width or 1

    self.valueAlignment = layoutTable.Value_Alignment or Turbine.UI.ContentAlignment.MiddleCenter
    self.sideAlignment = layoutTable.Side_Alignment or Turbine.UI.ContentAlignment.MiddleLeft
    self.topAlignment = layoutTable.Side_Alignment or Turbine.UI.ContentAlignment.MiddleCenter

    self.totalCellWidth = self.lineWidth * 2 + self.cellWidth
    self.totalCellHeight = self.lineWidth * 2 + self.cellHeight


    self.cells = {}
    self.lines = {}

    self.tableListbox = Turbine.UI.ListBox()
    self.tableListbox:SetParent(self)
    self.tableListbox:SetSize(self:GetWidth() - 10, self:GetHeight() - 10)
    self.tableListbox:SetMaxColumns(self.columns)
    self.tableListbox:SetMaxRows(self.rows)
    self.tableListbox:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.tableVerticalScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.tableVerticalScrollbar:SetParent(self)
    self.tableVerticalScrollbar:SetSize(10, self:GetHeight() - 10)
    self.tableVerticalScrollbar:SetPosition(self:GetWidth() - 10, 0)
    self.tableVerticalScrollbar:SetOrientation(Turbine.UI.Orientation.Vertical)

    self.tableHorizontalScrollbar = Turbine.UI.Lotro.ScrollBar()
    self.tableHorizontalScrollbar:SetParent(self)
    self.tableHorizontalScrollbar:SetSize(self:GetWidth() - 10, 10)
    self.tableHorizontalScrollbar:SetPosition(0, self:GetHeight() - 10)
    self.tableHorizontalScrollbar:SetOrientation(Turbine.UI.Orientation.Horizontal)

    self.tableListbox:SetVerticalScrollBar(self.tableVerticalScrollbar)
    self.tableListbox:SetHorizontalScrollBar(self.tableHorizontalScrollbar)


    if data then
        if headerData then

            local emptyCell = Cell(self, layoutTable, true)
            emptyCell:SetText(names or "")
            self.tableListbox:AddItem(emptyCell)

            for key, value in pairs(headerData) do
                local cell = Cell(self, layoutTable)
                cell:SetText(key)
                cell:SetAlignment(self.topAlignment)
                self.tableListbox:AddItem(cell)
            end
        end

        for key, value in pairs(data) do
            local cell = Cell(self, layoutTable, true)
            cell:SetText(key)
            cell:SetAlignment(self.sideAlignment)
            self.tableListbox:AddItem(cell)
            for key2, value2 in pairs(value) do
                local cell2 = Cell(self, layoutTable)
                cell2:SetText(value2)
                cell2:SetAlignment(self.valueAlignment)
                self.tableListbox:AddItem(cell2)
            end
        end
    else
        for i = 1, self.rows do
            if not self.cells[i] then self.cells[i] = {} end
            for j = 1, self.columns do
                self.tableListbox:AddItem( Cell(self, layoutTable))
            end
        end

    end
end

Cell = class(Turbine.UI.Control)
function Cell:Constructor(parent, layoutTable, first)
    Turbine.UI.Control.Constructor(self)

    self.lineColour = layoutTable.Line_Colour or Turbine.UI.Color.Black
    self.cellColour = layoutTable.Cell_Colour or Turbine.UI.Color.White

    self.cellWidth = first and 300 or layoutTable.Cell_Width or 120
    self.cellHeight = layoutTable.Cell_Height or 20

    self.lineWidth = layoutTable.Line_Width or 1

    self.valueAlignment = layoutTable.Value_Alignment or Turbine.UI.ContentAlignment.MiddleCenter

    self:SetParent(parent)
    self:SetSize(self.lineWidth * 2 + self.cellWidth, self.lineWidth * 2 + self.cellHeight)
    self:SetBackColor(self.lineColour)

    self.label = Turbine.UI.Label()
    self.label:SetParent(self)
    self.label:SetSize(self.cellWidth, self.cellHeight)
    self.label:SetPosition(self.lineWidth, self.lineWidth)
    self.label:SetBackColor(self.cellColour)
    self.label:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft)
    self.label:SetForeColor(Turbine.UI.Color.Black)
end

function Cell:SetText(text)
    self.label:SetText(tostring(text))
end

function Cell:SetAlignment(alignment)
    self.label:SetTextAlignment(alignment)
end